/**
 * 
 * @author Christian Rempis
 * c.rempis@gmx.de
 * 
 */
package de.uos.ikw.octavio.spinalcordmanager.apps;



import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.sql.Date;
import java.text.SimpleDateFormat;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JRadioButtonMenuItem;

import de.uos.ikw.octavio.spinalcordmanager.core.SpinalCordManager;
import de.uos.ikw.octavio.spinalcordmanager.gui.FrameContainer;
import de.uos.ikw.octavio.spinalcordmanager.gui.NetworkActivationPanel;
//import de.uos.ikw.octavio.spinalcordmanager.gui.OscillationTestPanel;
import de.uos.ikw.octavio.spinalcordmanager.gui.byteObserver.ByteObserver;
import de.uos.ikw.octavio.spinalcordmanager.gui.connector.ConnectionControlPanel;
import de.uos.ikw.octavio.spinalcordmanager.gui.deployment.DeploymentPanel;
import de.uos.ikw.octavio.spinalcordmanager.gui.historypanel.HistoryPanel;
import de.uos.ikw.octavio.spinalcordmanager.gui.legcontrol.LegControlPanel;
import de.uos.ikw.octavio.spinalcordmanager.gui.logpanel.LogPanel;
import de.uos.ikw.octavio.spinalcordmanager.gui.plotter.GraphPlotter;
import de.uos.ikw.octavio.spinalcordmanager.gui.rawcommandpanel.RawCommandPanel;
import de.uos.ikw.octavio.spinalcordmanager.gui.sensormapping.SensorMappingPanel;
import de.uos.ikw.octavio.spinalcordmanager.gui.tableviewer.IgnoreSynchronizationSelectionPanel;
import de.uos.ikw.octavio.spinalcordmanager.gui.tableviewer.TableViewer;
import de.uos.ikw.octavio.spinalcordmanager.gui.util.ArrangeEEPROMFramesAction;
import de.uos.ikw.octavio.spinalcordmanager.gui.util.ArrangeFramesAction;

/**
 * @author chris
 *
 */
public class TestApplication {

	
	public TestApplication(String[] args) {
		SpinalCordManager scm = SpinalCordManager.getInstance();
		scm.setupManager();
		scm.parseCommandLineArguments(args);
		
		scm.getDataManager().readInitializationFile("Default");
		//scm.getDataManager().setLogFile("Logdata.txt");
		//scm.getDataManager().setLogData(true);
		
		JFrame frame = new JFrame("Spinal Cord Test Application");
		frame.setSize(800, 600);
		
		frame.addWindowListener(new WindowAdapter() {

			public void windowClosing(WindowEvent e) {
				super.windowClosing(e);
				SpinalCordManager.getInstance().shutDown();
				System.exit(0);
			}
			
		});
		
		
		
		JPanel mainPanel = (JPanel) frame.getContentPane();
		mainPanel.setLayout(new BorderLayout());
		
		
		TableViewer tableViewer = new TableViewer("TableView");
		mainPanel.add(tableViewer.getDisplay(), BorderLayout.CENTER);
		
		ConnectionControlPanel connectionPanel = new ConnectionControlPanel();
		mainPanel.add(connectionPanel.getDisplay(), BorderLayout.NORTH);
		
		JPanel lowerPanel = new JPanel(new BorderLayout());
		
		HistoryPanel historyPanel = new HistoryPanel();
		lowerPanel.add(historyPanel.getDisplay(), BorderLayout.NORTH);
		
		NetworkActivationPanel networkActivationPanel = new NetworkActivationPanel();
		JPanel lowerRightPanel = new JPanel(new BorderLayout());
		lowerRightPanel.add(networkActivationPanel.getDisplay(), BorderLayout.CENTER);
		lowerPanel.add(lowerRightPanel, BorderLayout.SOUTH);
		
		mainPanel.add(lowerPanel, BorderLayout.SOUTH);
		
		LegControlPanel legController = new LegControlPanel("LegControlPanel");
		FrameContainer legFrame = new FrameContainer(legController.getDisplay(), "Leg Control", "Leg Controller", 400, 200);
		
		RawCommandPanel rawComPanel = new RawCommandPanel();
		FrameContainer rawComFrame = new FrameContainer(rawComPanel.getDisplay(), "Raw Command", "Raw Command Panel", 800, 100);
		
		GraphPlotter plotter = new GraphPlotter("GraphPlotter");
		FrameContainer plotFrame = new FrameContainer(plotter.getDisplay(), "Value Plotter", "Value Plotter", 800, 250);
		tableViewer.addCellSelectionListener(plotter);
		
		SensorMappingPanel sensorMappingPanel = new SensorMappingPanel();
		FrameContainer sensorMappingFrame = new FrameContainer(sensorMappingPanel.getDisplay(), "Sensor Mapping", "Sensor Mapping", 420, 200);
		tableViewer.addCellSelectionListener(sensorMappingPanel);
				
		LogPanel logPanel = new LogPanel();
		FrameContainer logPanelFrame = new FrameContainer(logPanel.getDisplay(), "Logger", "Logger", 400, 400);
		
		//OscillationTestPanel osciPanel = new OscillationTestPanel();
		//FrameContainer osciPanelFrame = new FrameContainer(osciPanel.getDisplay(), "Oscillation Test", "OscillationTest", 500, 150);
		
		IgnoreSynchronizationSelectionPanel issPanel = new IgnoreSynchronizationSelectionPanel();
		FrameContainer ignoreSynchronizationFrame = new FrameContainer(issPanel.getDisplay(), "Synchronization Control", "Synchronization Control", 200, 60);
		
		ByteObserver byteObserver = new ByteObserver();
		FrameContainer byteObserverFrame = new FrameContainer(byteObserver.getDisplay(), "Raw Byte Observer", "Raw Byte Observer", 600, 500);
		
		DeploymentPanel deploymentPanel = new DeploymentPanel();
		FrameContainer deploymentFrame = new FrameContainer(deploymentPanel.getDisplay(), "Deployment", "Deployment", 550, 600);
		
		//Util
		JPanel utilPanel = new JPanel(new BorderLayout());
		
		ArrangeFramesAction arranger = new ArrangeFramesAction();
		arranger.setMainFrame(frame);
		arranger.setLoggerFrame(logPanelFrame);
		arranger.setMappingFrame(sensorMappingFrame);
		arranger.setPlotFrame(plotFrame);
		//lowerRightPanel.add(new JButton(arranger), BorderLayout.EAST);
		
		ArrangeEEPROMFramesAction eepromArranger = new ArrangeEEPROMFramesAction();
		eepromArranger.setMainFrame(frame);
		eepromArranger.setDeploymentFrame(deploymentFrame);
		eepromArranger.setMappingFrame(sensorMappingFrame);
		eepromArranger.setPlotFrame(plotFrame);
		eepromArranger.setRawByteObserverFrame(byteObserverFrame);
		eepromArranger.setRawCommandFrame(rawComFrame);
		
		
		utilPanel.add(BorderLayout.EAST, new JButton(arranger));
		utilPanel.add(BorderLayout.WEST, new JButton(eepromArranger));
		
		lowerRightPanel.add(utilPanel, BorderLayout.EAST);
		
		//Menu
		JMenuBar menubar = new JMenuBar();
		frame.setJMenuBar(menubar);
		JMenu controlMenu = new JMenu("Control");
		menubar.add(controlMenu);
		JRadioButtonMenuItem kernelVerboseRadioButton = new JRadioButtonMenuItem("Kernel Verbose");
		kernelVerboseRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(((JRadioButtonMenuItem)e.getSource()).isSelected())
				{
					SpinalCordManager.getInstance().getCommandWriter().setVerbose(true);
				}
				else
				{
					SpinalCordManager.getInstance().getCommandWriter().setVerbose(false);
				}
			}
		});
		
		JRadioButtonMenuItem logFileRadioButton = new JRadioButtonMenuItem("Log Data");
		logFileRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(((JRadioButtonMenuItem)e.getSource()).isSelected())
				{
					Date date = new Date(System.currentTimeMillis());
					SimpleDateFormat df = new SimpleDateFormat("yyyy.MM_dd.hh.mm.ss");
					String fileName = "LogData_" 
										+ df.format(date) 
										+ ".txt";
					System.err.println("Starting to log data to file [" + fileName + "]");
					SpinalCordManager.getInstance().getDataManager().setLogFile(fileName);
					SpinalCordManager.getInstance().getDataManager().setLogData(true);
				}
				else
				{
					SpinalCordManager.getInstance().getDataManager().setLogData(false);
					System.err.println("Stopping to log data to file.");
				}
			}
		});
		
		controlMenu.add(plotFrame.getTrigger());
		controlMenu.add(legFrame.getTrigger());
		controlMenu.add(sensorMappingFrame.getTrigger());
		//controlMenu.add(osciPanelFrame.getTrigger());
		controlMenu.addSeparator();
		controlMenu.add(rawComFrame.getTrigger());
		controlMenu.add(deploymentFrame.getTrigger());
		controlMenu.add(byteObserverFrame.getTrigger());
		controlMenu.addSeparator();
		controlMenu.add(logPanelFrame.getTrigger());
		controlMenu.add(ignoreSynchronizationFrame.getTrigger());
		controlMenu.addSeparator();
		controlMenu.add(arranger);
		controlMenu.add(eepromArranger);
		controlMenu.addSeparator();
		controlMenu.add(kernelVerboseRadioButton);
		controlMenu.add(logFileRadioButton);
		
		scm.init();
		
		frame.setVisible(true);
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		new TestApplication(args);
	}

}
