/**
 * 
 * @author Christian Rempis
 * c.rempis@gmx.de
 * 
 */
package de.uos.ikw.octavio.spinalcordmanager.core;

import java.awt.Color;
import java.util.Hashtable;
import java.util.logging.Logger;


import de.uos.ikw.octavio.spinalcordmanager.formatter.DataFormatter;

public class OctavioDataStructure {
	
	
	
	public void setUpOctavioDataStructure(DataManager dataManager) {
		
		int columns = 9;
		int rows = 27;
		
		dataManager.setDimensions(columns, rows);
		dataManager.setSlotSizes(8, 32, 48);
		dataManager.setFrameNumberIndex(4);
		
		addMasterSlot(dataManager);
		int index = rows;
		for(int i = 0; i < (columns - 1); ++i) {
			addSlot(dataManager, index);
			index += rows;
		}
	}
	
	private void addMasterSlot(DataManager dataManager) {
		Hashtable<String, DataFormatter> formatters = dataManager.getAvailableDataFormatters();
		
		//Formatters
		
		DataFormatter sync =  formatters.get("Sync");
		DataFormatter int8 = formatters.get("Int8");
		DataFormatter int16 = formatters.get("Int16");
		DataFormatter empty0 = formatters.get("Empty0");
		DataFormatter normDouble = formatters.get("NormalizedDouble");
		
		if(sync == null || int8 == null || int16 == null || normDouble == null || empty0 == null) {
			Logger.getLogger("").warning("Could not find all required DataFormatter objects!");
		}
		
		dataManager.setDataFormatter(0, sync.createCopy());
		dataManager.setDataFormatter(1, sync.createCopy());	
		dataManager.setDataFormatter(2, int16.createCopy());
		dataManager.setDataFormatter(3, int16.createCopy());
		dataManager.setDataFormatter(4, int8.createCopy());
		dataManager.setDataFormatter(5, int8.createCopy());
		dataManager.setDataFormatter(6, empty0.createCopy());
		dataManager.setDataFormatter(7, empty0.createCopy());
		dataManager.setDataFormatter(8, empty0.createCopy());
		
		dataManager.setDataFormatter(9, sync.createCopy());    	//sync
		dataManager.setDataFormatter(10, int8.createCopy());  	//id
		dataManager.setDataFormatter(11, int8.createCopy());  	//master ack
		dataManager.setDataFormatter(12, int8.createCopy()); 	//STATUS
		dataManager.setDataFormatter(13, int16.createCopy());
		for(int i = 0; i < 13; ++i) {
			dataManager.setDataFormatter(14 + i, normDouble.createCopy());
		}
		
		//Colors
		Hashtable<String, Color> mColors = dataManager.getAvailableDataColors();
		
		Color cyan = mColors.get("cyan");
		Color red = mColors.get("red");
		Color orange = mColors.get("orange");
		Color white = mColors.get("white");
		Color gray = mColors.get("gray");
		
		int i = 0;
		for(; i < 6; ++i) {
			dataManager.setDataColor(i, red);
		}
		for(; i < 9; ++i) {
			dataManager.setDataColor(i, gray);
		}
		for(; i < 13; ++i) {
			dataManager.setDataColor(i, orange);
		}
		dataManager.setDataColor(i, white);
		i++;
		for(; i < 27; ++i) {
			dataManager.setDataColor(i, cyan);
		}
	}
	
	private void addSlot(DataManager dataManager, int startPos) {
		
		Hashtable<String, DataFormatter> formatters = dataManager.getAvailableDataFormatters();
		
		
		//Formatters
		
		DataFormatter sync =  formatters.get("Sync");
		DataFormatter int8 = formatters.get("Int8");
		DataFormatter normDouble = formatters.get("NormalizedDouble");
		
		if(sync == null || int8 == null || normDouble == null) {
			Logger.getLogger("").warning("Could not find all required DataFormatter objects!");
		}


		dataManager.setDataFormatter(startPos + 0, sync.createCopy());     	//sync
		dataManager.setDataFormatter(startPos + 1, int8.createCopy());  	//id
		dataManager.setDataFormatter(startPos + 2, int8.createCopy());  	//master ack
		dataManager.setDataFormatter(startPos + 3, int8.createCopy()); 		//STATUS
		dataManager.setDataFormatter(startPos + 4, int8.createCopy()); 		//Command ack
		dataManager.setDataFormatter(startPos + 5, int8.createCopy()); 		//Reserved
		for(int i = 0; i < 21; ++i) {
			dataManager.setDataFormatter(startPos + 6 + i, normDouble.createCopy());
		}
		
		
		//Colors
		Hashtable<String, Color> mColors = dataManager.getAvailableDataColors();
		
		Color cyan = mColors.get("cyan");
		Color orange = mColors.get("orange");
		Color yellow = mColors.get("yellow");
		
		int i = startPos; 
		for(; i < startPos + 6; ++i) {
			dataManager.setDataColor(i, orange);
		}
		for(; i < startPos + 17; ++i) {
			dataManager.setDataColor(i, yellow);
		}
		for(; i < startPos + 27; ++i) {
			dataManager.setDataColor(i, cyan);
		}
	}

}
