/**
 * 
 * @author Christian Rempis
 * c.rempis@gmx.de
 * 
 */
package de.uos.ikw.octavio.spinalcordmanager.formatter;

public class DataFormatter {

	/** Member Variables *********************************************************
	 *****************************************************************************/

	
	public static final int MODE_VALUE = 0;
	public static final int MODE_BYTE = 1;
	public static final int MODE_HEX = 2;
	
	protected int mMode;
	private String mName;
	private String mDescription;
	private int mBytesPerData;


	/** Constructors *************************************************************
	 *****************************************************************************/
	
	public DataFormatter(String name, int bytesPerData) {
		this(name, bytesPerData, "");
	}
	
	public DataFormatter(String name, int bytesPerData, String description) {
		mMode = MODE_VALUE;
		mName = name;
		mDescription = description;
		mBytesPerData = bytesPerData;
	}


	public DataFormatter createCopy() {
		return new DataFormatter(mName, mBytesPerData, mDescription);
	}
		
	/** Public ************************************************************
	 *****************************************************************************/
	
	public String getName() {
		return mName;
	}
	
	public String getDescription() {
		return mDescription;
	}
	
	public void setDescription(String description) {
		mDescription = description;
	}
	
	public void setFormatMode(int mode) {
		mMode = mode;
	}
	
	public int getFormatMode() {
		return mMode;
	}
	
	public Object getFormattedData(byte[] data, int offset) {
		switch(mMode) {
			case MODE_VALUE:
				return formatData(data, offset);
			case MODE_BYTE:
				return formatAsBytes(data, offset);
			case MODE_HEX:
				return formatAsHex(data, offset);
			default:	
		}
		return null;
	}

	public Object formatData(byte[] data, int offset) {
		return null;
	}
	
	public Object formatAsBytes(byte[] data, int offset) {
		int numberOfBytes = getNumberOfBytesPerData();
		if(offset + numberOfBytes > data.length || offset < 0) {
			return null;
		}
		String value = "";
		for(int i = 0; i < numberOfBytes; ++i) {
			value += ((0xFF & (int) data[offset + i])) + "  ";
		}
		return value;
	}
	
	public Object formatAsHex(byte[] data, int offset) {
		int numberOfBytes = getNumberOfBytesPerData();
		if(offset + numberOfBytes > data.length || offset < 0) {
			return null;
		}
		String value = "";
		for(int i = 0; i < numberOfBytes; ++i) {
			value += Integer.toHexString((0xFF & (int) data[offset + i])) + "  ";
		}
		return value;
	}
	
	public int getNumberOfBytesPerData() {
		return mBytesPerData;
	}

		
	/** Public *******************************************************************
	 *****************************************************************************/
		
		
		
		
		
	/** Protected ****************************************************************
	 *****************************************************************************/
		
		
		
		
		
	/** Private *******************************************************************
	 *****************************************************************************/
		




	/** Internal Classes *********************************************************
	 *****************************************************************************/





	/** Main Method **************************************************************
	 *****************************************************************************/
	 
	 
	
}
