package de.uos.ikw.octavio.spinalcordmanager.formatter;

public class NormalizedDoubleFormatter extends DataFormatter implements PlotDataSource {

	
	private double mMinValue;
	private double mMaxValue;
	private double mMinNormalized;
	private double mMaxNormalized;
	//private boolean mInvertNormalized;
	
	public static final int MODE_MIN = 3;
	public static final int MODE_MAX = 4;
	public static final int MODE_MINMAX = 5;
	public static final int MODE_NORMALIZED_MINMAX = 6;
	public static final int MODE_IGNORE_NORMALIZATION = 7;
	
	public NormalizedDoubleFormatter() {
		super("NormalizedDouble", 2);
		
		mMinValue = 0.0;
		mMaxValue = 65536.0;
		mMinNormalized = 0.0;
		mMaxNormalized = 1.0;
		//mInvertNormalized = false;
		
	}
	
	public DataFormatter createCopy() {
		NormalizedDoubleFormatter formatter = new NormalizedDoubleFormatter();
		formatter.mMinValue = mMinValue;
		formatter.mMaxValue = mMaxValue;
		formatter.mMinNormalized = mMinNormalized;
		formatter.mMaxNormalized = mMaxNormalized;
		//formatter.mInvertNormalized = mInvertNormalized;
		
		return formatter;
	}
	
	public Object getFormattedData(byte[] data, int offset) {
		switch(mMode) {
			case MODE_MIN:
				return formatMin();
			case MODE_MAX:
				return formatMax();
			case MODE_MINMAX:
				return formatMinMax();
			case MODE_NORMALIZED_MINMAX:
				return formatNormalizedMinMax();
			case MODE_IGNORE_NORMALIZATION:
				return formatIntIgnoringMapping(data, offset);
			default:	
		}
		return super.getFormattedData(data, offset);
	}

	public Object formatData(byte[] data, int offset) {
		if(offset + 1 >= data.length || offset < 0) {
			return null;
		}
		return new Double(getAsDouble(data, offset));
	}
	
	public Object formatMax() {
		return new Integer((int)mMaxValue);
	}
	
	public Object formatMin() {
		return new Integer((int)mMinValue);
	}
	
	public Object formatMinMax() {
		return "[" + (int) mMinValue + "," + (int) mMaxValue + "]";
	}
	
	public Object formatNormalizedMinMax() {
		return "[" + mMinNormalized + "," + mMaxNormalized + "]";
	}
	
	public Object formatIntIgnoringMapping(byte[] data, int offset) {
		if(offset + 1 >= data.length || offset < 0) {
			return new Integer(0);
		}
		return new Integer(((int) ((0xFF & (int) data[offset + 1]) << 8 
									| (0xFF & (int) data[offset]))));		
	}
	
	public double getAsDouble(byte[] data, int offset) {
		if(offset + 1 >= data.length || offset < 0) {
			return 0.0;
		}
		double min = (mMinValue <= mMaxValue) ? mMinValue : mMaxValue;
		double max = (mMaxValue >= mMinValue) ? mMaxValue : mMinValue;
		
		double value = 0.0;
		if(min != max) {
			value = (double) ((int) ((0xFF & (int) data[offset + 1]) << 8 | (0xFF & (int) data[offset])));
			value = Math.min(max, Math.max(min, value));

			if(max - min == 0.0) {
				value = 0.0;
			}
			else {
				value = (mMinNormalized + (((mMaxNormalized - mMinNormalized) 
					/ (max - min)) * (value - min)));
				if(Double.isNaN(value)) {
					return 0.0;
				}
			}
		}
		if(isInverted()) {
			value = mMinNormalized + (mMaxNormalized - value);
		}
		return value;
	}
	
	
	public double getUnnormalizedDouble(byte[] data, int offset) {
		if(offset + 1 >= data.length || offset < 0) {
			return 0.0;
		}
		
		return (double) ((int) ((0xFF & (int) data[offset + 1]) << 8 | (0xFF & (int) data[offset])));

	}
	
	public void setRange(double min, double max) {
		if(min == Double.MAX_VALUE && max != Double.MIN_VALUE) {
			min = max;
		}
		else if(max == Double.MIN_VALUE && min != Double.MAX_VALUE) {
			max = min;
		}
		mMinValue = min;
		mMaxValue = max;
	}
	
	public double getMin() {
		return mMinValue;
	}
	
	public double getMax() {
		return mMaxValue;
	}
	
	public void setNormalizedRange(double min, double max) {
		mMinNormalized = min;
		mMaxNormalized = max;
	}
	
	public double getMinNormalized() {
		return mMinNormalized;
	}
	
	public double getMaxNormalized() {
		return mMaxNormalized;
	}
	
//	public void setInvertNormalized(boolean invert) {
//		mInvertNormalized = invert;
//	}
	
	public boolean isInverted() {
		//return mInvertNormalized;
		return (mMinValue > mMaxValue);
	}
	
}
