package de.uos.ikw.octavio.spinalcordmanager.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JPanel;

import de.uos.ikw.octavio.spinalcordmanager.core.SpinalCordManager;
import de.uos.ikw.octavio.spinalcordmanager.io.Command;
import de.uos.ikw.octavio.spinalcordmanager.io.CommandWriter;
import de.uos.ikw.octavio.spinalcordmanager.io.ErrorStreamListener;

public class NetworkActivationPanel extends GuiElement implements ErrorStreamListener {

	private JPanel mDisplay;
	private JButton mStartNetworkModeButton;
	private JButton mEndNetworkModeButton;
	private JComboBox mTargetSelection;
	
	public NetworkActivationPanel() {
		super("NetworkActivationPanel");
		
		mDisplay = new JPanel(new BorderLayout());
		mStartNetworkModeButton = new JButton("Networks ON");
		mEndNetworkModeButton = new JButton("Networks OFF");
		mDisplay.setBackground(Color.red);
		
		mTargetSelection = new JComboBox();
		mTargetSelection.addItem(new String("Linux Kernel"));
		mTargetSelection.addItem(new String("Octavio EEPROM"));
		mTargetSelection.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				handleTargetChange();				
			}
		});
		
		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		buttonPanel.add(mTargetSelection);
		buttonPanel.add(mStartNetworkModeButton);
		buttonPanel.add(mEndNetworkModeButton);
		
		mDisplay.add(buttonPanel, BorderLayout.WEST);
		//mDisplay.add(new JPanel(), BorderLayout.CENTER);
		
		mStartNetworkModeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int target = CommandWriter.NETWORK_TARGET_KERNEL;
				
				if(mTargetSelection.getSelectedIndex() == 1) { //hard coded!
					target = CommandWriter.NETWORK_TARGET_EEPROM;
				}
				if(target == CommandWriter.NETWORK_TARGET_EEPROM) {
					//enforece sensor mapping
					int[] message = new int[4];
					message[0] = Command.nextCommandId();
					message[1] = 255;
					message[2] = 80; //sensor mapping on
					message[3] = 0;
					for(int i = 0; i < 2; ++i) { //TODO check why command has to be sent twice
						SpinalCordManager.getInstance().getCommandWriter()
						  .sendCommand(message, false,	false, -1, false);
					}
					
					mDisplay.setBackground(Color.green);
				}
				SpinalCordManager.getInstance().getCommandWriter().setNetworkMode(true, target);
			}
		});
		
		mEndNetworkModeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				CommandWriter writer = SpinalCordManager.getInstance().getCommandWriter();
				
				int target = CommandWriter.NETWORK_TARGET_KERNEL;
				
				if(mTargetSelection.getSelectedIndex() == 1) { //hard coded!
					target = CommandWriter.NETWORK_TARGET_EEPROM;
				}
				
				writer.setNetworkMode(false, target);
				
				if(target == CommandWriter.NETWORK_TARGET_EEPROM) {
					mDisplay.setBackground(Color.red);
				}
				
				//Send emergency break command
				int command[] = new int[10];
				
				command[0] = Command.nextCommandId();
				command[1] = 255; //all motors
				command[2] = 0x70; //Leg Motor Command
				command[3] = 6;
				
				command[4] = 3;
				command[5] = 255;
				command[6] = 3;
				command[7] = 255;
				command[8] = 3;
				command[9] = 255;

				//verify, do not discard
				writer.sendCommand(command, true, false, -1, false); 
			}
		});
		
		
		SpinalCordManager.getInstance().getErrorReader().addErrorStreamListener(this);
	}
	

	public JPanel getDisplay() {
		return mDisplay;
	}
	

	public void receivedErrorMessage(String errorMessage) {
		if(errorMessage.startsWith("[N0]")) {
			mDisplay.setBackground(Color.red);
		}
		else if(errorMessage.startsWith("[N1]")) {
			mDisplay.setBackground(Color.green);
		}
	}
	
	public void receivedDataMessage(String messageHeader, byte[] data) {
		// TODO Auto-generated method stub
		
	}
	
	public void handleTargetChange() {
		
		CommandWriter writer = SpinalCordManager.getInstance().getCommandWriter();
		
		writer.setNetworkMode(false, CommandWriter.NETWORK_TARGET_KERNEL);
		writer.setNetworkMode(false, CommandWriter.NETWORK_TARGET_EEPROM);
		
		mDisplay.setBackground(Color.red);
	}
	
}
