/**
 * 
 * @author Christian Rempis
 * c.rempis@gmx.de
 * 
 */
package de.uos.ikw.octavio.spinalcordmanager.gui.connector;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.logging.Logger;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JTextField;

import de.uos.ikw.octavio.spinalcordmanager.core.SpinalCordManager;
import de.uos.ikw.octavio.spinalcordmanager.core.SystemObject;
import de.uos.ikw.octavio.spinalcordmanager.gui.GuiElement;
import de.uos.ikw.octavio.spinalcordmanager.io.CommandWriter;

public class ConnectionControlPanel extends GuiElement implements SystemObject {

	private JPanel mDisplay;
	private JTextField mFileNameField;
	private JComboBox mTypeChooser;
	private JButton mConnectButton;
	private static final String mFileMode = "File";
	private static final String mOnlineMode = "Online";
	private static final String mDeviceMode = "Device";
	private Process mOnlineStreamProcess;
	private FileInputStream mBusDeviceInput;
	private FileOutputStream mBusDeviceOutput;
	private FileInputStream mConfigDeviceInput;
	
	public ConnectionControlPanel() {
		super("ConnectionControlPanel");
		
		SpinalCordManager.getInstance().addSystemObject(this);
		
		
		mDisplay = new JPanel(new BorderLayout());
		
		mFileNameField = new JTextField("./SpinalCordManager.exe");

		Object[] modes = {mOnlineMode, mFileMode, mDeviceMode};
		mTypeChooser = new JComboBox(modes);

		String operatingSystemName = System.getProperty("os.name").toLowerCase();
		if(!operatingSystemName.contains("windows")) {
			mFileNameField.setText("<default>");
			mTypeChooser.setSelectedItem(mDeviceMode);
		}
		
		mConnectButton = new JButton("Connect");
		mConnectButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mConnectButton.getText().equals("Connect")) {
					
					if(mTypeChooser.getSelectedItem().toString().equals(mFileMode)) {
						if(mOnlineStreamProcess != null) {
							mOnlineStreamProcess.destroy();
						}
						
						File  file = new File(mFileNameField.getText());
						if(file.isDirectory() || !file.exists() || !file.canRead()) {
							Logger.getLogger("").warning("Could not open desired input file!");
							return;
						}
						try {
							InputStream is = new FileInputStream(file);
							SpinalCordManager.getInstance().getDataReader().setInputStream(is, true);
						} catch (FileNotFoundException e1) {
							return;
						}
					}
					else if(mTypeChooser.getSelectedItem().toString().equals(mOnlineMode)) {
						if(mOnlineStreamProcess != null) {
							mOnlineStreamProcess.destroy();
						}
						
						String[] command = new String[2];
						//command[0] = "C:\\programming\\cpp\\SpinalCordManager\\Debug\\SpinalCordManager.exe";
						command[0] = mFileNameField.getText();
						command[1] = "v";
						//command[2] = "r";
						//command[2] = "f test.txt";

						try {
							mOnlineStreamProcess = Runtime.getRuntime().exec(command);
							
							SpinalCordManager sm = SpinalCordManager.getInstance();
							
							sm.getDataReader().setInputStream(mOnlineStreamProcess.getInputStream(), false);

							sm.getCommandWriter().setOutputType(CommandWriter.MODE_CHARACTER);
							sm.getCommandWriter().setOutputStream(mOnlineStreamProcess.getOutputStream());
							sm.getErrorReader().setSerialLine(mOnlineStreamProcess.getErrorStream());
							
						} catch (IOException ex) {
							System.out.println("Could not connect to specified interface programm! \n" + mFileNameField.getText());
						}
					}
					else if(mTypeChooser.getSelectedItem().toString().equals(mDeviceMode)){
						if(mOnlineStreamProcess != null) {
							mOnlineStreamProcess.destroy();
						}

						try {
							mBusDeviceInput = new FileInputStream("/dev/octoBus");
							mBusDeviceOutput = new FileOutputStream("/dev/octoBus");
							mConfigDeviceInput = new FileInputStream("/dev/octoConf");
							
							SpinalCordManager sm = SpinalCordManager.getInstance();
							sm.getDataReader().setInputStream(mBusDeviceInput, false);
							sm.getCommandWriter().setOutputType(CommandWriter.MODE_BYTE);
							sm.getCommandWriter().setOutputStream(mBusDeviceOutput);
							sm.getErrorReader().setSerialLine(mConfigDeviceInput);
							
						} catch (FileNotFoundException e1) {
							//e1.printStackTrace();
							Logger.getLogger("").warning("Connector Error: Could not open device /dev/octoBus or /dev/octoConf!");
							return;
						}
					}
					
					mConnectButton.setText("Disconnect");
				}
				else {
					
					SpinalCordManager sm = SpinalCordManager.getInstance();
					sm.getDataReader().setInputStream(null, false);
					sm.getErrorReader().setSerialLine(null);
					sm.getCommandWriter().setOutputStream(null);
					
					if(mOnlineStreamProcess != null) {
						mOnlineStreamProcess.destroy();
						mOnlineStreamProcess = null;
					}
					if(mBusDeviceInput != null) {
						try {
							mBusDeviceInput.close();
						} catch (IOException e1) {
						}
						mBusDeviceInput = null;
					}
					if(mBusDeviceOutput != null) {
						try {
							mBusDeviceOutput.close();
						} catch (IOException e1) {
						}
						mBusDeviceOutput = null;
					}
					if(mConfigDeviceInput != null) {
						try {
							mConfigDeviceInput.close();
						} catch (IOException e1) {
						}
						mConfigDeviceInput = null;
					}
					
					mConnectButton.setText("Connect");
				}
			}
		});
		
		mDisplay.add(mFileNameField, BorderLayout.CENTER);
		mDisplay.add(mTypeChooser, BorderLayout.WEST);
		mDisplay.add(mConnectButton, BorderLayout.EAST);
		
		Runtime.getRuntime().addShutdownHook(new Thread() {
			public void run() {
				if(mOnlineStreamProcess != null) {
					mOnlineStreamProcess.destroy();
					mOnlineStreamProcess = null;
				}
			}
			
		});
	}
	
	
	public JPanel getDisplay() {
		return mDisplay;
	}

	public JTextField getFileNameField() {
		return mFileNameField;
	}


	public boolean init() {
		return true;
	}


	public boolean shutDown() {
		if(mOnlineStreamProcess != null) {
			mOnlineStreamProcess.destroy();
			mOnlineStreamProcess = null;
		}
		return true;
	}
	
	
}
