package de.uos.ikw.octavio.spinalcordmanager.gui.deployment;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextArea;
import javax.swing.JTextField;


import de.uos.ikw.octavio.spinalcordmanager.core.SpinalCordManager;
import de.uos.ikw.octavio.spinalcordmanager.gui.GuiElement;
import de.uos.ikw.octavio.spinalcordmanager.io.Command;
import de.uos.ikw.octavio.spinalcordmanager.io.CommandWriter;

public class DeploymentPanel extends GuiElement {
	
	public static final int DISPLAY_MODE_BYTE = 1;
	public static final int DISPLAY_MODE_WORD = 2;

	private JPanel mDisplay;
	private JButton mStartDeploymentButton;
	private JButton mEndDeploymentButton;
	private JTextField mDataFileField;
	private JFileChooser mDataFileChooser;
	private JButton mFileChooserButton;
	private JTextField mEEPROMAddressField;
	private JButton mWriteButton;
	private JButton mReadButton;
	private JButton mUploadNetButton;
	private JTextArea mByteArea;
	private JSpinner mLegNumber;
	private JTextField mNumberBytesToRead;
	private JTextField mMaxNumberOfBytesPerMessage;
	private DeploymentReaderThread mReaderThread;
	private DeploymentWriterThread mWriterThread;
	private int mDisplayMode = DISPLAY_MODE_WORD;
	private JCheckBox mShowWords;
	private boolean mNetworkLoadingMode;
	
	public DeploymentPanel() {
		super("DeploymentPanel");
		
		mNetworkLoadingMode = false;
		
		mDisplay = new JPanel(new BorderLayout());
		mDisplayMode = DISPLAY_MODE_BYTE;
		
		mStartDeploymentButton = new JButton("Start Deployment");
		mStartDeploymentButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				sendDeploymentCommand();
			}
		});
		
		mEndDeploymentButton = new JButton("End Deployment");
		mEndDeploymentButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				sendEndDeploymentCommand();
			}
		});
		
		
		JPanel upperButtonPanel = new JPanel(new GridLayout(0, 2));
		upperButtonPanel.add(mStartDeploymentButton);
		upperButtonPanel.add(mEndDeploymentButton);
		
		mDataFileField = new JTextField("");
		mDataFileField.setEditable(false);
		
		mEEPROMAddressField = new JTextField("61440");
		mNumberBytesToRead = new JTextField("1000");
		mMaxNumberOfBytesPerMessage = new JTextField("20");
		mLegNumber = new JSpinner();
		mLegNumber.setValue(new Integer(1));
		
		JPanel addressPanel = new JPanel(new GridLayout(4, 2));
		addressPanel.add(new JLabel("Leg number:"));
		addressPanel.add(mLegNumber);
		addressPanel.add(new JLabel("EEPROM start adress: "));
		addressPanel.add(mEEPROMAddressField);
		addressPanel.add(new JLabel("Number of bytes to read:"));
		addressPanel.add(mNumberBytesToRead);
		addressPanel.add(new JLabel("Max data size:"));
		addressPanel.add(mMaxNumberOfBytesPerMessage);
		
		addressPanel.setBorder(BorderFactory.createTitledBorder("Deployment Settings: "));
		
		mFileChooserButton = new JButton("Load File");
		mFileChooserButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				loadBytesFromFile();
			}
		});
		
		mWriteButton = new JButton("Write");
		mWriteButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				sendWriteEEPROMCommand();
			}
		});
		
		mReadButton = new JButton("Read");
		mReadButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				mNetworkLoadingMode = false;
				int numberOfBytesToRead = 0;
				try {
					numberOfBytesToRead= Integer.parseInt(mNumberBytesToRead.getText());
				} catch (NumberFormatException nfe) {
					System.out.println("Could not read number of bytes to read. Was no Integer. " 
							+ mNumberBytesToRead.getText());
					return;
				}
				sendReadEEPROMCommand(numberOfBytesToRead);
			}
		});
		
		mUploadNetButton = new JButton("Upload Net");
		mUploadNetButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				mNetworkLoadingMode = true;

				sendReadEEPROMCommand(58);
			}
		});
		
		mShowWords = new JCheckBox("Words");
		mShowWords.setSelected(false);
		mShowWords.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Vector<Integer> data = getBytesFromByteArea();
				if(mShowWords.isSelected()) {
					mDisplayMode = DISPLAY_MODE_WORD;
				}
				else {
					mDisplayMode = DISPLAY_MODE_BYTE;
				}
				updateByteArea(data);
			}
		});
		
		JPanel readWritePanel = new JPanel(new GridLayout(1, 3));
		readWritePanel.add(mShowWords);
		readWritePanel.add(mReadButton);
		readWritePanel.add(mWriteButton);
		readWritePanel.add(mFileChooserButton);
		readWritePanel.add(mUploadNetButton);

		
		Box box =  Box.createVerticalBox();
		box.add(upperButtonPanel);
		box.add(addressPanel);
		box.add(readWritePanel);
		
		mByteArea = new JTextArea();
		JScrollPane pane = new JScrollPane(mByteArea);
		
		mDisplay.add(BorderLayout.NORTH, box);
		mDisplay.add(BorderLayout.CENTER, pane);
		
		mDataFileChooser = new JFileChooser();
	
	}

	@Override
	public JPanel getDisplay() {
		return mDisplay;
	}
	

	private void sendDeploymentCommand() {
		
		int command[] = new int[4];
		
		
		command[0] = Command.nextCommandId();
		command[1] = 255;	
		command[2] = 32; //Deployment Command
		command[3] = 0;

		CommandWriter cw = SpinalCordManager.getInstance().getCommandWriter();
		
		cw.setAutoRepeat(false);
		
		try {
			Thread.sleep(100);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		//has to be sent twice.
		cw.sendCommand(command, false, false, -1, false); 
		cw.sendCommand(command, false, false, -1, false);
	}
	
	private void sendEndDeploymentCommand() {
		int[] dataBytes = new int[1];
		dataBytes[0] = (int) 0x04;
		
		CommandWriter cw = SpinalCordManager.getInstance().getCommandWriter();
		cw.sendCommand(dataBytes, false, false, -1, true); 
	}
	
//	private boolean sendClearEEPROMCommand() {
//		int[] dataBytes = new int[3];
//		dataBytes[0] = (int) 'C';
//		try {
//			dataBytes[1] = Integer.parseInt(mLegNumber.getValue().toString());
//		} catch (NumberFormatException e) {
//			return false;
//		}
//		dataBytes[2] = (int) 0x17;
//		
//		CommandWriter cw = SpinalCordManager.getInstance().getCommandWriter();
//		cw.sendCommand(dataBytes, false, false, -1, true); 
//		
//		//TODO wait for reply.
//		
//		
//		return true;
//	}
	
	public void sendReadEEPROMCommand(int size) {
		int startAddress = 0;
		try {
			startAddress= Integer.parseInt(mEEPROMAddressField.getText());
		} catch (NumberFormatException e) {
			System.out.println("Could not read EEPROM start address. Was no Integer. " 
					+ mEEPROMAddressField.getText());
			return;
		}
		int maxDataSize = 0;
		try {
			maxDataSize= Integer.parseInt(mMaxNumberOfBytesPerMessage.getText());
		} catch (NumberFormatException e) {
			System.out.println("Could not read maximal bytes per message. Was no Integer. " 
					+ mMaxNumberOfBytesPerMessage.getText());
			return;
		}
		
		
		mReaderThread = new DeploymentReaderThread(this, ((Integer) mLegNumber.getValue()).intValue(), 
				startAddress, size, maxDataSize);
		mWriteButton.setEnabled(false);
		mReadButton.setEnabled(false);
		mUploadNetButton.setEnabled(false);
		mFileChooserButton.setEnabled(false);
		mShowWords.setEnabled(false);
		
		SpinalCordManager.getInstance().getDataReader().addRawByteListener(mReaderThread);
		
		Thread threadRunner = new Thread(mReaderThread);
		threadRunner.start();
	}
	
	public void notifyReadCompleted(boolean success) {		
		SpinalCordManager.getInstance().getDataReader().removeRawByteListener(mReaderThread);
		
		updateByteArea(mReaderThread.getReadData());
		
//		if(!success) {
//			Vector<Integer> data = new Vector<Integer>();
//			for(int i = 0; i < 100; ++i) {
//				data.add(i);
//			}
//			updateByteArea(data);
//			success = true;
//		}
		if(mNetworkLoadingMode && success) {
			if(uploadNet()) {
				mNetworkLoadingMode = false;
				return;
			}
		}
		mWriteButton.setEnabled(true);
		mReadButton.setEnabled(true);
		mUploadNetButton.setEnabled(true);
		mFileChooserButton.setEnabled(true);
		mShowWords.setEnabled(true);

	}
	
	
	public void sendWriteEEPROMCommand() {
		Vector<Integer> data = getBytesFromByteArea();
		
		if(data.isEmpty()) {
			System.out.println("There was no data to write to the EEPROM. Skipping.");
			return;
		}
		
		int maxDataSize = 0;
		try {
			maxDataSize= Integer.parseInt(mMaxNumberOfBytesPerMessage.getText());
		} catch (NumberFormatException e) {
			System.out.println("Could not read maximal bytes per message. Was no Integer. " 
					+ mMaxNumberOfBytesPerMessage.getText());
			return;
		}
		
		mWriterThread = new DeploymentWriterThread(this, ((Integer) mLegNumber.getValue()).intValue(),
				maxDataSize, data);
		mWriteButton.setEnabled(false);
		mReadButton.setEnabled(false);
		
		SpinalCordManager.getInstance().getDataReader().addRawByteListener(mWriterThread);
		
		Thread threadRunner = new Thread(mWriterThread);
		threadRunner.start();
	}
	
	public void notifyWriteCompleted(boolean success) {
		SpinalCordManager.getInstance().getDataReader().removeRawByteListener(mWriterThread);
		
		System.out.println("Data written successfully. " + success);
		
		mWriteButton.setEnabled(true);
		mReadButton.setEnabled(true);
		mUploadNetButton.setEnabled(true);
		mFileChooserButton.setEnabled(true);
		mShowWords.setEnabled(true);
	}
	
	public void loadBytesFromFile() {
		int decision = mDataFileChooser.showOpenDialog(mDisplay);
		if(decision != JFileChooser.APPROVE_OPTION) {
			return;
		}
		File file = mDataFileChooser.getSelectedFile();
		if(!file.exists() || file.isDirectory()) {
			return;
		}
		Vector<Integer> data = new Vector<Integer>();
		try {
			FileInputStream input = new FileInputStream(file);
			while(input.available() > 0) {
				int value = input.read();
				data.add(value);
			}
			input.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return;
		} catch (IOException e) {
			e.printStackTrace();
			return;
		}
		updateByteArea(data);
	}
	
	public boolean uploadNet() {
		int decision = mDataFileChooser.showOpenDialog(mDisplay);
		if(decision != JFileChooser.APPROVE_OPTION) {
			return false;
		}
		File file = mDataFileChooser.getSelectedFile();
		if(!file.exists() || file.isDirectory()) {
			return false;
		}
		Vector<Integer> eepromData = getBytesFromByteArea();
		
		Vector<Integer> data = new Vector<Integer>();
		
		for(int i = 0; i < 58 && i < eepromData.size(); ++i) {
			data.add(eepromData.elementAt(i));
		}
		
		try {
			FileInputStream input = new FileInputStream(file);
			while(input.available() > 0) {
				int value = input.read();
				data.add(value);
			}
			input.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return false;
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}
		updateByteArea(data);
		
		sendWriteEEPROMCommand();
		
		return true;
	}
	
	public void updateByteArea(Vector<Integer> bytes) {
		StringBuffer buffer = new StringBuffer();
		buffer.append("[0\t] ");
		int numberOfValues = 0;
		for(int i = 0; i < bytes.size(); ++i) {
			if(mDisplayMode == DISPLAY_MODE_BYTE) {
				buffer.append(bytes.elementAt(i));
			}
			else {
				int first = bytes.elementAt(i);
				++i;
				if(i < bytes.size()) {
					buffer.append((int) ((0xFF & (int) bytes.elementAt(i)) << 8 | (0xFF & (int) first)));
				}
				else {
					buffer.append((int) (0xFF & (int) first));
				}
			}
			numberOfValues++;
			
			if(numberOfValues % 10 == 0)  {
				buffer.append("\n[" + numberOfValues + "\t] ");
			}
			else {
				buffer.append(",");
			}
		}
		mByteArea.setText(buffer.toString());
	}
	
	public Vector<Integer> getBytesFromByteArea() {
		Vector<Integer> bytes = new Vector<Integer>();
		
		StringBuffer text = new StringBuffer(mByteArea.getText());
		while(text.indexOf("[") != -1) {
			int index = text.indexOf("[");
			int endIndex = text.indexOf("]", index);
			if(endIndex == -1) {
				endIndex = text.length();
			}
			else {
				endIndex++;
			}
			text.delete(index, endIndex);
		}
		
		String[] dataStrings = text.toString().replace("\n", ",").split(",");
		for(int i = 0; i < dataStrings.length; ++i) {
			try {
				if(dataStrings[i].trim().equals("")) {
					continue;
				}
				int data = Integer.parseInt(dataStrings[i].trim());
				if(mDisplayMode == DISPLAY_MODE_BYTE) {
					while(data > 255) {
						data -= 256;
					}
					while(data < 0) {
						data += 256;
					}
					bytes.add(data);
				}
				else {
					int first = (0xFF & data);
					int second = (0xFF & (data >> 8));
					while(first > 255) {
						first -= 256;
					}
					while(first < 0) {
						first += 256;
					}
					while(second > 255) {
						second -= 256;
					}
					while(second < 0) {
						second += 256;
					}
					bytes.add(first);
					bytes.add(second);
				}
			} catch (NumberFormatException e) {
				System.out.println("Could not parse [" + dataStrings[i] + "].");
				bytes.clear();
				return bytes;
			}
		}
		
		return bytes;
	}
	
	
}
