package de.uos.ikw.octavio.spinalcordmanager.gui.deployment;

import java.util.Vector;


import de.uos.ikw.octavio.spinalcordmanager.core.SpinalCordManager;
import de.uos.ikw.octavio.spinalcordmanager.io.CommandWriter;

public class DeploymentReaderThread extends DeploymentThread {

	private int mNumberOfBytesToRead;
	private int mMaximalDataSize;
	private int mLegNumber;
	private Vector<Integer> mReadData = new Vector<Integer>();
	private int mCurrentReadPosition;
	private DeploymentPanel mOwner;
	
	public DeploymentReaderThread(DeploymentPanel owner, 
			int legNumber, int startAddress, int numberOfBytesToRead, int maxDataSize) 
	{
		mOwner = owner;
		mLegNumber = legNumber;
		mNumberOfBytesToRead = numberOfBytesToRead;
		mMaximalDataSize = maxDataSize;
		mCurrentReadPosition = startAddress;
	}
	
	public void run() {
		System.out.println("Starting to read EEPROM");
		boolean success = readEEPROM();
		
		mOwner.notifyReadCompleted(success);
		System.out.println("Finished reading EEPROM " + success);
	}
	
	private boolean readEEPROM() {
		mReadData.clear();
		
		CommandWriter cw = SpinalCordManager.getInstance().getCommandWriter();
		
		
		for(int i = 0; i < mNumberOfBytesToRead;) {
			int lowerByte = (int) (0xFF & mCurrentReadPosition >> 8);
			int higherByte = (int) (0xFF & (mCurrentReadPosition));

			int length = Math.min(mMaximalDataSize, mNumberOfBytesToRead - i);
			
			int message[] = new int[7];
			message[0] = (int) 'R';
			message[1] = mLegNumber;
			message[2] = higherByte;
			message[3] = lowerByte;
			message[4] = length;
			message[5] = calculateChecksum(message, 1, 5);
			message[6] = (int) 0x17;
			
			printArray(message);
			
			i += mMaximalDataSize;
			mCurrentReadPosition += mMaximalDataSize;
			
			clearData();
			cw.sendCommand(message, false, false, -1, true); 
			
			if(!waitForReply()) {
				System.out.println("Failed to read EEPROM.");
				return false;
			}
			else {
				printArray(toIntArray(mData));
				
				synchronized (mData) {
					if(mData.elementAt(0).intValue() == DeploymentThread.COMMAND_NACK) {
						System.out.println("Error reading EEPROM: NACK received after READ command.");
						return false;
					}
					else if(!(mData.elementAt(0).intValue() == DeploymentThread.COMMAND_READREPLY)
						|| mData.size() != (7 + length)) 
					{
						System.out.println("Failed to read EEPROM: Wrong reply detected. Command: "
								+ mData.elementAt(0) + " Length: " + mData.size());
						return false;
					}
					else {
						if(mData.elementAt(1).intValue() != mLegNumber) {
							System.out.println("Failed to read EEPROM: Wrong leg number received: "
									+ mData.elementAt(1).intValue());
							return false;
						}
						if(mData.elementAt(2).intValue() != higherByte
							|| mData.elementAt(3).intValue() != lowerByte)
						{
							System.out.println("Failed to read EEPROM: Wrong address received: "
									+ mData.elementAt(2).intValue() + " " 
									+ mData.elementAt(3).intValue());
							return false;
						}
						if(mData.elementAt(4).intValue() != length) {
							System.out.println("Failed to read EEPROM: Wrong length received: " 
									+ mData.elementAt(4).intValue());
						}
						if(mData.elementAt(6 + length).intValue() != (int) 0x17) {
							System.out.println("Failed to read EEPROM: Termination symbol wrong: "
									+ mData.elementAt(6 + length).intValue());
							return false;
						}
//						if(mData.elementAt(5 + mMaximalDataSize).intValue() 
//								!= calculateChecksum(toIntArray(mData), 1, 5 + mMaximalDataSize)
//								)
//						{
//							System.out.println("Failed to read EEPROM: Checksum wrong. Is "
//									+ mData.elementAt(5 + mMaximalDataSize) + " Expected: "
//									+ calculateChecksum(toIntArray(mData), 1, 5 + mMaximalDataSize));
//							return false;
//						}
						System.out.println("Read message was ok!");
						for(int j = 0; j < length; ++j) {
							mReadData.add(mData.elementAt(5 + j));
						}
					}
				}
			}
		}
		System.out.println("Entire READ procedure successful.");
		return true;
	}
	
	public Vector<Integer> getReadData() {
		return mReadData;
	}

}
