package de.uos.ikw.octavio.spinalcordmanager.gui.deployment;

import java.util.Vector;

import de.uos.ikw.octavio.spinalcordmanager.io.RawByteListener;


public abstract class DeploymentThread implements Runnable, RawByteListener {
	
	public static final int COMMAND_READ = (int) 'R';
	public static final int COMMAND_READREPLY = (int) 'A';
	public static final int COMMAND_CLEAR = (int) 'C';
	public static final int COMMAND_SEND = (int) 'M';
	public static final int COMMAND_ACK = (int) 0x06;
	public static final int COMMAND_NACK = (int) 0x15;
	

	protected Vector<Integer> mData = new Vector<Integer>();
	
	public DeploymentThread() {
	}
	
	public void clearData() {
		mData.clear();
	}
	
	public boolean waitForReply() {
		
		for(int i = 0; i < 5000; ++i) {
			synchronized(mData) {
				if(mData.size() > 0) {
					
					int commandByte = mData.elementAt(0).intValue();
					System.out.println("datasize > 0 : " + commandByte);
					if(commandByte == COMMAND_ACK) {
						return true;
					}
					else if(commandByte == COMMAND_NACK) {
						return true;
					}
					else if(commandByte == COMMAND_READREPLY) {
						System.out.println("command read");
						if(mData.size() >= 7) {
							System.out.println("data size > 7");
							int size = mData.elementAt(4).intValue();
							System.out.println("size " + size);
							if(mData.size() == size + 7) {
								System.out.println("Found READ message");
								return true;
							}
						}
					}
				}
			}
			try {
				synchronized(this) {
					wait(2);
				}
			} catch (InterruptedException e) {
			}
		}
		System.out.println("FAILED waiting for reply: TIMEOUT");
		return false;
	}

	
	public int calculateChecksum(int[] message, int offset, int numberOfBytes) {
		int sum = 0;
		for(int i = offset; i < message.length && i < numberOfBytes; ++i) {
			sum += message[i];
			while(sum > 255) {
				sum -= 256;
			}
		}
		sum = (int) (0xFF & (~(0xFF & sum)));
		return sum;
	}
	
	public void printArray(int[] message) {
		System.out.print("Message: ");
		for(int i = 0; i < message.length; ++i) {
			System.out.print("" + message[i] + ",");
		}
		System.out.println("");
	}
	
	public int[] toIntArray(Vector<Integer> message) {
		int reply[] = new int[message.size()];
		for(int i = 0; i < message.size(); ++i) {
			reply[i] = message.elementAt(i).intValue();
		}
		return reply;
	}


	public void byteReceived(byte receivedByte) {
		System.out.println("got " + receivedByte);
		synchronized (mData) {
			int data = (int) (0xFF & (int) receivedByte);
			mData.add(data);
		}
		synchronized(this) {
			notifyAll();
		}
	}
	
	
}
