package de.uos.ikw.octavio.spinalcordmanager.gui.deployment;

import java.util.Vector;

import de.uos.ikw.octavio.spinalcordmanager.core.SpinalCordManager;
import de.uos.ikw.octavio.spinalcordmanager.io.CommandWriter;

public class DeploymentWriterThread extends DeploymentThread {
	
	private int mMaximalDataSize;
	private int mLegNumber;
	private Vector<Integer> mDataToWrite = new Vector<Integer>();
	private int mCurrentWritePosition;
	private DeploymentPanel mOwner;
	
	public DeploymentWriterThread(DeploymentPanel owner, 
			int legNumber, int maxDataSize, Vector<Integer> dataToWrite)
	{
		mOwner = owner;
		mLegNumber = legNumber;
		mMaximalDataSize = maxDataSize;
		mDataToWrite = dataToWrite;
	}
	
	public void run() {
		System.out.println("Starting to write EEPROM");
		boolean success = writeEEPROM();
		
		mOwner.notifyWriteCompleted(success);
		System.out.println("Finished writing EEPROM " + success);
	}
	
	public boolean writeEEPROM() {
		
		mCurrentWritePosition = 61440;
		
		CommandWriter cw = SpinalCordManager.getInstance().getCommandWriter();
		
		//clear EEPROM
		{
			int[] clearCommand = new int[3];
			clearCommand[0] = (int) 'C';
			clearCommand[1] = mLegNumber;
			clearCommand[2] = (int) 0x17;
			
			clearData();
			cw.sendCommand(clearCommand, false, false, -1, true); 
		}
		
		if(!waitForReply()) {
			System.out.println("Failed to write EEPROM (Clear failed).");
			return false;
		}
		synchronized (mData) {
			if(mData.elementAt(0).intValue() != DeploymentThread.COMMAND_ACK) {
				System.out.println("Failed to write EEPROM (Clear reply was not ACK)");
				return false;
			}
		}

		
		for(int i = 0; i < mDataToWrite.size();) {
			int lowerByte = (int) (0xFF & mCurrentWritePosition >> 8);
			int higherByte = (int) (0xFF & (mCurrentWritePosition));

			int dataLength = Math.min(mMaximalDataSize, mDataToWrite.size() - i);
			
			int message[] = new int[7 + dataLength];
			message[0] = (int) 'M';
			message[1] = mLegNumber;
			message[2] = higherByte;
			message[3] = lowerByte;
			message[4] = dataLength;
			for(int j = 0; j < dataLength; ++j) {
				message[5 + j] = mDataToWrite.elementAt(i + j).intValue();
			}
			message[5 + dataLength] = calculateChecksum(message, 1, 5 + dataLength);
			message[6 + dataLength] = (int) 0x17;
			
			printArray(message);
			
			i += dataLength;
			mCurrentWritePosition += dataLength;
			
			clearData();
			cw.sendCommand(message, false, false, -1, true); 
			
			if(!waitForReply()) {
				System.out.println("Failed to write EEPROM.");
				return false;
			}
			synchronized (mData) {
				if(mData.elementAt(0).intValue() != DeploymentThread.COMMAND_ACK) {
					System.out.println("Failed to write EEPROM. Reply on data message was not ACK. " + 
							mData.elementAt(0));
				}
			}
		}
		
		System.out.println("Entire WRITE procedure successful.");
		return true;
		
	}

}
