/**
 * 
 * @author Christian Rempis
 * c.rempis@gmx.de
 * 
 */
package de.uos.ikw.octavio.spinalcordmanager.gui.historypanel;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JToggleButton;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uos.ikw.octavio.spinalcordmanager.core.DataFrame;
import de.uos.ikw.octavio.spinalcordmanager.core.DataManager;
import de.uos.ikw.octavio.spinalcordmanager.core.SpinalCordManager;
import de.uos.ikw.octavio.spinalcordmanager.gui.GuiElement;
import de.uos.ikw.octavio.spinalcordmanager.io.SerialDataReader;

public class HistoryPanel extends GuiElement {
	
	private JPanel mDisplay;
	private JToggleButton mActivateButton;
	private JLabel mCurrentFrameLabel;
	private JButton mPreviousButton;
	private JButton mNextButton;
	private Vector<DataFrame> mHistory;
	private int mCurrentIndex;
	private JSlider mFrameSlider;
	private boolean mAdjustingSlider;
	private JButton mHistorySizeButton;
	private JToggleButton mReplayButton;
	private JSlider mDelaySlider;
	private Timer mReplayTimer;

	public HistoryPanel() {
		super("HistoryPanel");
		
		mDisplay = new JPanel(new BorderLayout());
		
		mActivateButton = new JToggleButton("History");
		mActivateButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mActivateButton.isSelected()) {
					
					SerialDataReader reader = SpinalCordManager.getInstance().getDataReader();
					reader.setSingleFrameMode(true);
					
					
					DataManager dm = SpinalCordManager.getInstance().getDataManager();
					dm.setAutoSetData(false);
					
					mHistory = new Vector<DataFrame>(dm.getDataHistory());
					
					mFrameSlider.setMinimum(0);
					mFrameSlider.setMaximum(mHistory.size() - 1);
					mCurrentIndex = mHistory.size() - 1;
					
					setSliderValue(mCurrentIndex);

					mPreviousButton.setEnabled(true);
					mNextButton.setEnabled(true);
					mFrameSlider.setEnabled(true);
					mReplayButton.setEnabled(true);
					
					synchronizeFrame();
				}
				else {
					DataManager dm = SpinalCordManager.getInstance().getDataManager();
					mHistory = null;
					dm.setAutoSetData(true);
					
					SerialDataReader reader = SpinalCordManager.getInstance().getDataReader();
					reader.setSingleFrameMode(false);
					
					mPreviousButton.setEnabled(false);
					mNextButton.setEnabled(false);
					mFrameSlider.setEnabled(false);
					mReplayButton.setEnabled(false);
					mReplayButton.setSelected(false);
					setReplayMode(false);
					
					mCurrentIndex = -1;
					
					synchronizeFrame();
				}
			}
		});
		
		mCurrentFrameLabel = new JLabel("0/0");
		mCurrentFrameLabel.setPreferredSize(new Dimension(100, 10));
		
		mPreviousButton = new JButton("<");
		mPreviousButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				mCurrentIndex = Math.max(0, mCurrentIndex - 1);
				synchronizeFrame();
			}
		});
		
		mNextButton = new JButton(">");
		mNextButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				mCurrentIndex = Math.min(mHistory.size() - 1, mCurrentIndex + 1);
				synchronizeFrame();
				
			}
		});
		
		
		
		mFrameSlider = new JSlider(JSlider.HORIZONTAL);
		mFrameSlider.setPreferredSize(new Dimension(100, 10));
		mFrameSlider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				if(mAdjustingSlider) {
					return;
				}
				mCurrentIndex = mFrameSlider.getValue();
				synchronizeFrame();
			}
			
		});
		
		mPreviousButton.setEnabled(false);
		mNextButton.setEnabled(false);
		mFrameSlider.setEnabled(false);
		
		mHistorySizeButton = new JButton("" + SpinalCordManager.getInstance().getDataManager().getMaxHistorySize());
		mHistorySizeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					//String newSizeString = JOptionPane.showInputDialog(mDisplay, "Select the history size:", "History Size", JOptionPane.INFORMATION_MESSAGE);
					String newSizeString = JOptionPane.showInputDialog(mDisplay, "Select the history size:", mHistorySizeButton.getText());
					int newSize = Integer.parseInt(newSizeString);
					SpinalCordManager.getInstance().getDataManager().setMaxHistorySize(newSize);
					mHistorySizeButton.setText("" + newSize);
				} catch (NumberFormatException e1) {
				}
			}
		});
		
		mReplayButton = new JToggleButton("Replay");
		mReplayButton.setEnabled(false);
		mReplayButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setReplayMode(mReplayButton.isSelected());
			}
		});
		
		mDelaySlider = new JSlider(1, 1000, 100);
		mDelaySlider.setPreferredSize(new Dimension(50, 10));
		mDelaySlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				int delay = mDelaySlider.getValue();

				if(mReplayTimer != null && mReplayButton.isSelected()) {
					mReplayTimer.cancel();
					mReplayTimer = new Timer();
					mReplayTimer.schedule(new ReplayTimerTask(), delay, delay);
				}
			}
		});
		
		
		Box box = Box.createHorizontalBox();
		box.add(mHistorySizeButton);
		box.add(new JLabel("Frame"));
		box.add(mFrameSlider);
		box.add(mPreviousButton);
		box.add(mNextButton);
		box.add(mReplayButton);
		box.add(new JLabel("Delay"));
		box.add(mDelaySlider);

		
		mDisplay.add(mActivateButton, BorderLayout.WEST);
		mDisplay.add(mCurrentFrameLabel, BorderLayout.CENTER);
		mDisplay.add(box, BorderLayout.EAST);
		
		mAdjustingSlider = false;
		mReplayTimer = null;
	}
	
	
	public JPanel getDisplay() {
		return mDisplay;
	}
	
	public void synchronizeFrame() {
		
		mHistorySizeButton.setText("" + SpinalCordManager.getInstance().getDataManager().getMaxHistorySize());
		if(mHistory == null) {
			mCurrentFrameLabel.setText("0/0");
		}
		else {
			mCurrentFrameLabel.setText("" + (mCurrentIndex + 1) + "/" + mHistory.size());
			if(mCurrentIndex >= 0 && mCurrentIndex < mHistory.size()) {
				SpinalCordManager.getInstance().getDataManager().setRawData(
						mHistory.elementAt(mCurrentIndex).getData());
				
				setSliderValue(mCurrentIndex);
			}
		}
		
	}
	
	private void setSliderValue(int value) {
		mAdjustingSlider = true;
		mFrameSlider.setValue(value);
		mAdjustingSlider = false;
	}
	
	private void setReplayMode(boolean enable) {

		if(mReplayButton.isSelected()) {
			mNextButton.setEnabled(false);
			mPreviousButton.setEnabled(false);
			int delay = mDelaySlider.getValue();
			if(mReplayTimer != null) {
				mReplayTimer.cancel();
				mReplayTimer = null;
			}
			mReplayTimer = new Timer();
			mReplayTimer.schedule(new ReplayTimerTask(), 0, delay);
		}
		else {
			if(mActivateButton.isSelected()) {
				mNextButton.setEnabled(true);
				mPreviousButton.setEnabled(true);
			}
			if(mReplayTimer != null) {
				mReplayTimer.cancel();
				mReplayTimer = null;
			}
		}
	}
	
	public class ReplayTimerTask extends TimerTask {
		
		public void run() {
			if(mReplayButton.isSelected()) {
				if(mHistory != null) {
					mCurrentIndex = Math.min(mHistory.size() - 1, mCurrentIndex + 1);
					synchronizeFrame();
				}
			}
		}
		
	}

}
