/**
 * 
 * @author Christian Rempis
 * c.rempis@gmx.de
 * 
 */
package de.uos.ikw.octavio.spinalcordmanager.gui.legcontrol;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uos.ikw.octavio.spinalcordmanager.core.SpinalCordManager;
import de.uos.ikw.octavio.spinalcordmanager.gui.GuiElement;
import de.uos.ikw.octavio.spinalcordmanager.io.Command;
import de.uos.ikw.octavio.spinalcordmanager.io.CommandWriter;

public class LegControlPanel extends GuiElement implements KeyListener {
	
	private JPanel mDisplay;
	private JLabel mMotor1Label;
	private JLabel mMotor2Label;
	private JLabel mMotor3Label;
	private JSlider mMotor1Slider;
	private JSlider mMotor2Slider;
	private JSlider mMotor3Slider;
	private JTextField mMotor1Velocity;
	private JTextField mMotor2Velocity;
	private JTextField mMotor3Velocity;
	private JSpinner mLegNumber;
	private JButton mSetButton;
	private JCheckBox mAutoSetCheckBox;
	private JButton mBrakeMotorsButton;
	private JButton mStopMotorsButton;
	private int mExpirationDate;
	private JButton mClearKernelCommandQueue;
	

	

	public LegControlPanel(String name) {
		super(name);
		
		mExpirationDate = 100;
		
		mDisplay = new JPanel(new BorderLayout());
		
		
		mLegNumber = new JSpinner();
		mSetButton =  new JButton("Set");
		mSetButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				sendMotorCommand();
			}
		});
		
		JLabel legNumberLabel = new JLabel("Leg Number: ");
		Box legBox = Box.createHorizontalBox();
		legBox.add(legNumberLabel);
		legBox.add(mLegNumber);
		
		mAutoSetCheckBox = new JCheckBox("AutoSet");
		mAutoSetCheckBox.setSelected(false);
		
		mClearKernelCommandQueue = new JButton("Clear Command Queue");
		mClearKernelCommandQueue.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				CommandWriter writer = SpinalCordManager.getInstance().getCommandWriter();
				writer.setAutoRepeat(false);
				try {
					Thread.sleep(250);
				} catch (InterruptedException e) {
				}
				writer.setAutoRepeat(true);
			}
		});
		
		mBrakeMotorsButton = new JButton("Emergency Brake");
		mBrakeMotorsButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				mMotor1Velocity.setText("0.0");
				mMotor2Velocity.setText("0.0");
				mMotor3Velocity.setText("0.0");
				mMotor1Slider.setValue(0);
				mMotor2Slider.setValue(0);
				mMotor3Slider.setValue(0);
				
				sendEmergencyBrake();
			}
		});
		
		mStopMotorsButton = new JButton("Stop All");
		mStopMotorsButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				mMotor1Velocity.setText("0.0");
				mMotor2Velocity.setText("0.0");
				mMotor3Velocity.setText("0.0");
				mMotor1Slider.setValue(0);
				mMotor2Slider.setValue(0);
				mMotor3Slider.setValue(0);
				
				sendEmergencyStop();
			}
		});
		
		Box buttonBox = Box.createVerticalBox();
		buttonBox.add(mClearKernelCommandQueue);
		buttonBox.add(mBrakeMotorsButton);
		buttonBox.add(mStopMotorsButton);
		buttonBox.add(legBox);
		buttonBox.add(mSetButton);
		buttonBox.add(mAutoSetCheckBox);
		
		
		//Motor1
		mMotor1Slider = new JSlider(JSlider.VERTICAL);
		mMotor1Slider.setMinimum(-100);
		mMotor1Slider.setMaximum(100);
		mMotor1Slider.setValue(0);
		mMotor1Slider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				mMotor1Velocity.setText(Double.toString(((double) mMotor1Slider.getValue()) / 100.0));
				handleChangeEvent();
			}
		});
		
		mMotor1Velocity = new JTextField("0.0");
		mMotor1Velocity.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				try {
					int value = (int) Math.round(Double.parseDouble(mMotor1Velocity.getText()) * 100.0);
					mMotor1Slider.setValue(value);
				} catch (NumberFormatException e1) {
					mMotor1Velocity.setText(Double.toString(((double) mMotor1Slider.getValue()) / 100.0));
				}
				handleChangeEvent();
			}
			
		});
		
		mMotor1Label = new JLabel("Motor1");

		Box motor1Panel = Box.createVerticalBox();
		motor1Panel.add(mMotor1Label);
		motor1Panel.add(mMotor1Slider);
		motor1Panel.add(mMotor1Velocity);
		

//		Motor2
		mMotor2Slider = new JSlider(JSlider.VERTICAL);
		mMotor2Slider.setMinimum(-100);
		mMotor2Slider.setMaximum(100);
		mMotor2Slider.setValue(0);
		mMotor2Slider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				mMotor2Velocity.setText(Double.toString(((double) mMotor2Slider.getValue()) / 100.0));
				handleChangeEvent();
			}
		});
		
		mMotor2Velocity = new JTextField("0.0");
		mMotor2Velocity.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				try {
					int value = (int) Math.round(Double.parseDouble(mMotor2Velocity.getText()) * 100.0);
					mMotor2Slider.setValue(value);
				} catch (NumberFormatException e1) {
					mMotor2Velocity.setText(Double.toString(((double) mMotor2Slider.getValue()) / 100.0));
				}
				handleChangeEvent();
			}
			
		});
		
		mMotor2Label = new JLabel("Motor2");

		Box motor2Panel = Box.createVerticalBox();
		motor2Panel.add(mMotor2Label);
		motor2Panel.add(mMotor2Slider);
		motor2Panel.add(mMotor2Velocity);
		
//		Motor3
		mMotor3Slider = new JSlider(JSlider.VERTICAL);
		mMotor3Slider.setMinimum(-100);
		mMotor3Slider.setMaximum(100);
		mMotor3Slider.setValue(0);
		mMotor3Slider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				mMotor3Velocity.setText(Double.toString(((double) mMotor3Slider.getValue()) / 100.0));
				handleChangeEvent();
			}
		});
		
		mMotor3Velocity = new JTextField("0.0");
		mMotor3Velocity.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				try {
					int value = (int) Math.round(Double.parseDouble(mMotor3Velocity.getText()) * 100.0);
					mMotor3Slider.setValue(value);
				} catch (NumberFormatException e1) {
					mMotor3Velocity.setText(Double.toString(((double) mMotor3Slider.getValue()) / 100.0));
				}
				handleChangeEvent();
			}
			
		});
		
		mMotor3Label = new JLabel("Motor3");

		Box motor3Panel = Box.createVerticalBox();
		motor3Panel.add(mMotor3Label);
		motor3Panel.add(mMotor3Slider);
		motor3Panel.add(mMotor3Velocity);
		
		
		
		Box box = Box.createHorizontalBox();
		box.add(motor1Panel);
		box.add(motor2Panel);
		box.add(motor3Panel);
		
		
		JPanel leftPanel = new JPanel(new BorderLayout());
		leftPanel.add(buttonBox, BorderLayout.SOUTH);
		
		mDisplay.add(box, BorderLayout.WEST);
		mDisplay.add(leftPanel, BorderLayout.EAST);
		
		mDisplay.addKeyListener(this);
		mMotor1Velocity.addKeyListener(this);
		mMotor2Velocity.addKeyListener(this);
		mMotor3Velocity.addKeyListener(this);
		mMotor1Slider.addKeyListener(this);
		mMotor2Slider.addKeyListener(this);
		mMotor3Slider.addKeyListener(this);
		mAutoSetCheckBox.addKeyListener(this);
		mBrakeMotorsButton.addKeyListener(this);
		mLegNumber.addKeyListener(this);
		for(int i = 0; i < mLegNumber.getComponentCount(); ++i) {
			mLegNumber.getComponent(i).addKeyListener(this);
			Component c = mLegNumber.getComponent(i);
			if(c instanceof JComponent) {
				for(int j = 0; j < ((JComponent) c).getComponentCount(); ++j) {
					((JComponent) c).getComponent(j).addKeyListener(this);
				}
			}
		}
		mSetButton.addKeyListener(this);
		mStopMotorsButton.addKeyListener(this);
	}
	
	
	public JPanel getDisplay() {
		return mDisplay;
	}
	
	
	
	public void keyPressed(KeyEvent e) {
		if(e.getKeyCode() == KeyEvent.VK_F8) {
			sendEmergencyStop();
			System.out.println("Emergency Stop!");
		}
	}
	
	


	public void keyReleased(KeyEvent e) {
	}


	public void keyTyped(KeyEvent e) {
	}


	public void handleChangeEvent() {
		if(mAutoSetCheckBox.isSelected()) {
			sendMotorCommand();
		}
	}
	
	public void sendEmergencyStop() {
		CommandWriter writer = SpinalCordManager.getInstance().getCommandWriter();
		
		int command[] = new int[4];
		
		command[0] = Command.nextCommandId();
		command[1] = 255;
		command[2] = 0x60; //Leg Motor Stop Command
		command[3] = 0;

		//verify, do not discard
		writer.sendCommand(command, true, false, -1, false);
		
		mMotor1Velocity.setText("0.0");
		mMotor2Velocity.setText("0.0");
		mMotor3Velocity.setText("0.0");
		mMotor1Slider.setValue(0);
		mMotor2Slider.setValue(0);
		mMotor3Slider.setValue(0);
	}
	
	public void sendEmergencyBrake() {
		CommandWriter writer = SpinalCordManager.getInstance().getCommandWriter();
		
		int command[] = new int[10];
		
		
		command[0] = Command.nextCommandId();
		//command[1] = ((Integer) mLegNumber.getValue()).intValue();
		command[1] = 255; //all motors
		//command[2] = 208; //Leg Motor Command
		command[2] = 0x70; //Leg Motor Command
		command[3] = 6;
		
		command[4] = 3;
		command[5] = 255;
		command[6] = 3;
		command[7] = 255;
		command[8] = 3;
		command[9] = 255;

		//verify, do not discard
		writer.sendCommand(command, true, false, -1, false);
	}
	
	public void sendMotorCommand() {
		CommandWriter writer = SpinalCordManager.getInstance().getCommandWriter();
		
		int command[] = new int[10];
		
		
		command[0] = Command.nextCommandId();
		command[1] = ((Integer) mLegNumber.getValue()).intValue();
		//command[2] = 208; //Leg Motor Command		
		command[2] = 0x70; //Leg Motor Command
		command[3] = 6;
		
		command[4] = getDirection(mMotor1Velocity);
		command[5] = getVelocity(mMotor1Velocity);
		command[6] = getDirection(mMotor2Velocity);
		command[7] = getVelocity(mMotor2Velocity);
		command[8] = getDirection(mMotor3Velocity);
		command[9] = getVelocity(mMotor3Velocity);

		//do not verify, discard if sending window was missed.
		writer.sendCommand(command, false, true, 
				SpinalCordManager.getInstance().getDataManager()
				.getLastReceivedFrameNumber() + mExpirationDate, false);
	}
	
	private int getVelocity(JTextField textField) {
		double motor = 0.0;
		try {
			motor = Math.abs(Double.parseDouble(textField.getText()));
		} catch (NumberFormatException e) {
		}
		
		return Math.min(255, Math.max((int) Math.round(motor * 255.0), 0));
	}
	
	private int getDirection(JTextField textField) {
		double motor = 0.0;
		try {
			motor = Double.parseDouble(textField.getText());
		} catch (NumberFormatException e) {
		}
		
		int dir = 0;
		if(motor > 0) {
			dir = 1;
		}
		else if(motor < 0) {
			dir = 2;
		}
		return dir;
	}

}
