/**
 * 
 * @author Christian Rempis
 * c.rempis@gmx.de
 * 
 */
package de.uos.ikw.octavio.spinalcordmanager.gui.plotter;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JToggleButton;

import de.uos.ikw.octavio.spinalcordmanager.core.DataListener;
import de.uos.ikw.octavio.spinalcordmanager.core.DataManager;
import de.uos.ikw.octavio.spinalcordmanager.core.SpinalCordManager;
import de.uos.ikw.octavio.spinalcordmanager.core.SystemObject;
import de.uos.ikw.octavio.spinalcordmanager.formatter.DataFormatter;
import de.uos.ikw.octavio.spinalcordmanager.formatter.PlotDataSource;
import de.uos.ikw.octavio.spinalcordmanager.gui.GuiElement;
import de.uos.ikw.octavio.spinalcordmanager.gui.tableviewer.CellSelectionListener;


public class GraphPlotter extends GuiElement implements DataListener, CellSelectionListener  {

	JPanel mDisplay;
	private Vector<Integer> mPlottedTableIndices = new Vector<Integer>();
	private Vector<Integer> mPlottedIndices = new Vector<Integer>();
	private Vector<LinkedList<Double>> mData = new Vector<LinkedList<Double>>();
	private Vector<String> mDataNames = new Vector<String>();
	private boolean mPlotFromHistory;
	private boolean mPlotMapped = true;
	private int mMaxDataBufferSize;
	private Vector<Color> mColors = new Vector<Color>();
	private DiagramPlot mPlotArea;
	private JButton mCenterViewButton;
	private JButton mCenterViewZeroButton;
	private JButton mCenterViewThousandButton;
	private JTextField mMaxHistoryField;
	private JTextField mScaleField;
	private JTextField mBaseOffsetField;
	private JCheckBox mPlotMappedCheckbox;
	
	private JToggleButton mAddButton;
	private JButton mRemoveAllButton;
	
	public GraphPlotter(String name) {
		super(name);
		SpinalCordManager.getInstance().getDataManager().addDataListener(this);
		mMaxDataBufferSize = 500;
		
		mColors.add(Color.GREEN);
		mColors.add(Color.RED);
		mColors.add(Color.BLUE);
		mColors.add(Color.BLACK);
		mColors.add(Color.CYAN);
		mColors.add(Color.MAGENTA);
		mColors.add(Color.YELLOW);
		mColors.add(Color.PINK);
		
		mDisplay = new JPanel(new BorderLayout());
		mPlotArea = new DiagramPlot(this);
		
		
		mAddButton = new JToggleButton("Add Cells");
		mRemoveAllButton = new JButton("Remove All");
		mRemoveAllButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				synchronized(mData) {
					mDataNames.clear();
					mData.clear();
					mPlottedIndices.clear();
					mPlottedTableIndices.clear();
					mPlotArea.repaint();
				}
			}
			
		});
		
		mCenterViewButton = new JButton("Y-Range [-1,1]");
		mCenterViewButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				mPlotArea.setVerticalOffset(0.0);
				mPlotArea.setScale(1.0);
				mPlotArea.repaint();
			}
			
		});
		
		mCenterViewZeroButton = new JButton("Y-Range [0,1]");
		mCenterViewZeroButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				mPlotArea.setVerticalOffset(0.5);
				mPlotArea.setScale(2);
				mPlotArea.repaint();
			}
			
		});
		
		mCenterViewThousandButton = new JButton("Y-Range [0,1000]");
		mCenterViewThousandButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				mPlotArea.setVerticalOffset(500);
				mPlotArea.setScale(0.002);
				mPlotArea.repaint();
			}
			
		});
		
		mMaxHistoryField = new JTextField("500");
		mMaxHistoryField.setMaximumSize(new Dimension(150, 25));
		mMaxHistoryField.setPreferredSize(new Dimension(150, 25));
		mMaxHistoryField.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				try {
					int historySize = Integer.parseInt(mMaxHistoryField.getText());
					mMaxDataBufferSize = historySize;
				} catch (NumberFormatException e1) {
					mMaxHistoryField.setText("" + mMaxDataBufferSize);
				}
			}
			
		});
		JLabel historyLabel = new JLabel("History Size");
		historyLabel.setPreferredSize(new Dimension(100, 50));
		JPanel historyPanel = new JPanel(new BorderLayout());
		historyPanel.add(historyLabel, BorderLayout.WEST);
		historyPanel.add(mMaxHistoryField, BorderLayout.CENTER);
		
		
		mScaleField = new JTextField("1.0");
		mScaleField.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				try {
					double scale = Double.parseDouble(mScaleField.getText());
					if(scale == mPlotArea.getScale()) {
						return;
					}
					mScaleField.setText("" + scale);
					mPlotArea.setScale(scale);
				} catch (NumberFormatException e1) {
					mScaleField.setText("" + mPlotArea.getScale());
				}
			}
			
		});
		JLabel scaleLabel = new JLabel("Scale");
		scaleLabel.setPreferredSize(new Dimension(100, 50));
		JPanel scalePanel = new JPanel(new BorderLayout());
		scalePanel.add(scaleLabel, BorderLayout.WEST);
		scalePanel.add(mScaleField, BorderLayout.CENTER);
		
		
		mBaseOffsetField = new JTextField("0.0");
		mBaseOffsetField.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				try {
					double offset = Double.parseDouble(mBaseOffsetField.getText());
					if(offset == mPlotArea.getVerticalOffset()) {
						return;
					}
					mBaseOffsetField.setText("" + offset);
					mPlotArea.setVerticalOffset(offset);
				} catch (NumberFormatException e1) {
					mBaseOffsetField.setText("" + mPlotArea.getVerticalOffset());
				}
			}
			
		});
		JLabel offsetLabel = new JLabel("Offset");
		offsetLabel.setPreferredSize(new Dimension(100, 50));
		JPanel offsetPanel = new JPanel(new BorderLayout());
		offsetPanel.add(offsetLabel, BorderLayout.WEST);
		offsetPanel.add(mBaseOffsetField, BorderLayout.CENTER);
		
		mPlotMappedCheckbox = new JCheckBox("map", mPlotMapped);
		mPlotMappedCheckbox.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				mPlotMapped = mPlotMappedCheckbox.isSelected();
			}
			
		});
		
		
		JPanel buttonBox = new JPanel(new GridLayout(9, 1));
		buttonBox.add(mAddButton);
		buttonBox.add(mRemoveAllButton);
		buttonBox.add(mCenterViewButton);
		buttonBox.add(mCenterViewZeroButton);
		buttonBox.add(mCenterViewThousandButton);
		buttonBox.add(historyPanel);
		buttonBox.add(scalePanel);
		buttonBox.add(offsetPanel);
		buttonBox.add(mPlotMappedCheckbox);

		buttonBox.setPreferredSize(new Dimension(150, 200));
		
		JPanel eastPanel = new JPanel(new BorderLayout());
		eastPanel.add(buttonBox, BorderLayout.NORTH);
		JPanel spacer = new JPanel(new BorderLayout());
		spacer.setPreferredSize(new Dimension(100, 500));
		eastPanel.add(spacer, BorderLayout.CENTER);
		
		mDisplay.add(mPlotArea, BorderLayout.CENTER);
		mDisplay.add(eastPanel, BorderLayout.EAST);
		
		mPlotArea.setScale(1.0);
		
	}



	public void selectionChanged(int[] selectedIndices) {
		if(mAddButton.isSelected()) {
			for(int i = 0; i < selectedIndices.length; ++i) {
				addPlottedCellIndex(selectedIndices[i]);
			}
		}
	}



	public boolean addPlottedCellIndex(int index) {
		
		DataFormatter[] formatters = SpinalCordManager.getInstance().getDataManager().getDataFormatters();
		
		if(index < 0 || index >= formatters.length) {
			return false;
		}
		
		if(formatters[index] == null
		 	|| !(formatters[index] instanceof PlotDataSource))
		{
			return false;
		}
		
		for(Iterator<Integer> i = mPlottedTableIndices.iterator(); i.hasNext(); ) {
			if(i.next().intValue() == index) {
				return false;
			}
		}
		
		//ok, this index seems to be a valid index for plotter data.
		mPlottedTableIndices.add(index);
		mDataNames.add("Cell " + index);
		mData.add(new LinkedList<Double>());
		
		int byteIndex = 0;
		for(int i = 0; i < index; ++i) {
			byteIndex += formatters[i].getNumberOfBytesPerData();
		}
		mPlottedIndices.add(byteIndex);
		
		rebuildPlot();
		
		return true;
	}
	
	@Override
	public JPanel getDisplay() {
		return mDisplay;
	}
	
	public void updatePlot() {
		mPlotArea.repaint();
	}
	
	public void rebuildPlot() {
		mPlotArea.repaint();
	}

	public void dataContentChanged() {
		synchronized (mData) {

			if(!mPlotFromHistory) {
				DataManager dm = SpinalCordManager.getInstance().getDataManager();
				
				DataFormatter[] formatters = dm.getDataFormatters();
				byte[] data = dm.getLastInterpretedDataFrame();
				if(data == null || data.length != dm.getNumberOfBytesPerFrame()) {
					return;
				}
				Iterator<Integer> i = mPlottedTableIndices.iterator();
				Iterator<LinkedList<Double>> d = mData.iterator();
				Iterator<Integer> bi = mPlottedIndices.iterator();
				for(; i.hasNext() && d.hasNext() && bi.hasNext();) {
					LinkedList<Double> plottedData = d.next();
					int index = i.next();
					if(mPlotMapped) {
						plottedData.addLast(((PlotDataSource) formatters[index]).getAsDouble(data, bi.next()));
					}
					else {
						plottedData.addLast(((Integer)((PlotDataSource) formatters[index]).formatIntIgnoringMapping(data, bi.next())).doubleValue());
					}
					while(plottedData.size() > mMaxDataBufferSize) {
						plottedData.removeFirst();
					}
				}
				updatePlot();
			}
		}
	}

	public void dataStructureChanged() {
		
		
	}

	public void dataHistoryChanged() {
		if(mPlotFromHistory) {
			DataManager dm = SpinalCordManager.getInstance().getDataManager();
			
			if(dm.getDataHistory().isEmpty()) {
				return;
			}
			
			DataFormatter[] formatters = dm.getDataFormatters();
			byte[] data = dm.getDataHistory().getLast().getData();
			if(data == null || data.length != dm.getNumberOfBytesPerFrame()) {
				return;
			}
			Iterator<Integer> i = mPlottedTableIndices.iterator();
			Iterator<LinkedList<Double>> d = mData.iterator();
			Iterator<Integer> bi = mPlottedIndices.iterator();
			for(; i.hasNext() && d.hasNext() && bi.hasNext();) {
				LinkedList<Double> plottedData = d.next();
				int index = i.next();
				if(mPlotMapped) {
					plottedData.addLast(((PlotDataSource) formatters[index]).getAsDouble(data, bi.next()));
				}
				else {
					plottedData.addLast(((Integer)((PlotDataSource) formatters[index]).formatIntIgnoringMapping(data, bi.next())).doubleValue());
				}
				while(plottedData.size() > mMaxDataBufferSize) {
					plottedData.removeFirst();
				}
			}
			updatePlot();
		}
	}
	
	
	
	
public class DiagramPlot extends JPanel implements SystemObject, MouseWheelListener, 
			MouseMotionListener, MouseListener 
{
		
		public static final long serialVersionUID = 123L; 
		private transient GraphPlotter mOwner;
		private boolean mRepaintPlot;
		private DiagramPlot mSelf;
		private Timer mTimer;
		private double mScale;
		private double mVerticalScale;
		private Point mClickPosition;
		private double mVerticalOffset;
		private double mVerticalOffsetRoot;
		
		public DiagramPlot(GraphPlotter owner) {
			mOwner = owner;
			setBorder(BorderFactory.createEtchedBorder());
			mRepaintPlot = true;
			mSelf = this;
			SpinalCordManager.getInstance().addSystemObject(this);
			addMouseMotionListener(this);
			addMouseWheelListener(this);
			addMouseListener(this);
		}
		
		public boolean init() {
			suspendPaintTimer();
			resumePaintTimer();
			return true;
		}	
		
		public boolean shutDown() {
			return true;
		}

		public void suspendPaintTimer() {
			if(mTimer != null) {
				mTimer.cancel();
				mTimer = null;
			}
		}
		
		public void resumePaintTimer() {
			if(mTimer == null) {
				mTimer = new Timer();
				mTimer.schedule(new PaintTimer(), 300, 300);
			}
		}
		
		public void markForRepaint() {
			mRepaintPlot = true;
		}

		protected void paintComponent(Graphics g) {
			Graphics2D g2d = (Graphics2D) g;
			Color c = g2d.getColor();
			
			synchronized(mOwner) {
				plot(g2d);
			}
			
			g2d.setColor(c);

		}
		
		public class PaintTimer extends TimerTask {
			
			public synchronized void run() {
				if(mRepaintPlot) {
					mRepaintPlot = false;
					mSelf.invalidate();
					mSelf.repaint();	
				}
			}	
		}
		
		private void plot(Graphics2D g2d) {
		
			int width = getWidth();
			int height = getHeight();
			
			//Clear area.
			g2d.setColor(Color.WHITE);
			g2d.fillRect(0, 0, width, height);
	
	
			//Draw the raster of the diagram.
			mVerticalScale = ((double) height - 20) / 2.0;
			g2d.setColor(Color.LIGHT_GRAY);
			g2d.drawLine(0, 9, width, 9);
			g2d.drawLine(0, (int) (mVerticalScale) + 10, width, (int) (1 * mVerticalScale) + 10);
			g2d.drawLine(0, (int) (2 * mVerticalScale) + 11, width, (int) (2 * mVerticalScale) + 11);
			
			
			g2d.drawLine(0, (int) (0.5 * mVerticalScale) + 10, width, (int) (0.5 * mVerticalScale) + 10);
			g2d.drawLine(0, (int) (1.5 * mVerticalScale) + 10, width, (int) (1.5 * mVerticalScale) + 10);
			
			//plot graph
			plotHistroy(g2d);
			
			//Plot the names of the neurons in the colors of the neuron.
			synchronized (mData) {
			
				for(int i = 0; i < mDataNames.size() && i < mDataNames.size(); ++i) {
					
					g2d.setColor((Color) mColors.elementAt(i % mColors.size()));
		
					String name = mDataNames.elementAt(i);
					int offset = Math.max(0, name.length() - 25);
					g2d.drawChars(name.toCharArray(), offset, name.length() - offset, 10, 20 + (15 * i));
				}
			}
			
			g2d.setColor(Color.BLACK);
			//Plot the range
			g2d.setColor(Color.BLACK);
			String one = Double.toString((double) ((1.0 / mScale)+ mVerticalOffset));
			String minusOne = Double.toString((double) ((-1.0 / mScale) + mVerticalOffset));
			String zero = Double.toString((double) ((0.0 / mScale) + mVerticalOffset));
			g2d.drawChars(one.toCharArray(), 0, Math.min(15, one.length()), 7, 13);
			g2d.drawChars(zero.toCharArray(), 0, Math.min(15, zero.length()), 7, (int) (mVerticalScale) + 14);
			g2d.drawChars(minusOne.toCharArray(), 0, Math.min(15, minusOne.length()), 7, (int) (2 * mVerticalScale) + 14);
		}
		
		
		public double translateValue(double value) {
			return (((((value - mVerticalOffset) * mScale) - 1.0) * -1.0) * mVerticalScale) + 10.0;
		}
		
		public void setScale(double scale) {
			mScale = scale;
			mScaleField.setText("" + mScale);
			repaint();
		}
		
		public double getScale() {
			return mScale;
		}
		
		public void setVerticalOffset(double offset) {
			mVerticalOffset = offset;
			//mVerticalOffsetRoot = offset;
			mBaseOffsetField.setText("" + mVerticalOffset);
			repaint();
		}
		
		public double getVerticalOffset() {
			return mVerticalOffset;
		}
		
		public void plotHistroy(Graphics2D g2d) {
			
			double pointDist = ((double) getWidth()) / (mMaxDataBufferSize);
			synchronized (mData) {
				for(int i = 0; i < mData.size(); i++) {
					LinkedList<Double> data = mData.elementAt(i);
					if(data.isEmpty()) {
						continue;
					}
					Color c = mColors.elementAt(i % mColors.size());
					ListIterator<Double> values = data.listIterator(data.size() - 1);
					
					g2d.setColor(c);				
					
					int pointId = 0;
					Point point = new Point(0, 0);
					for(;values.hasPrevious();) {
						Double d = (Double) values.previous();
						
						Point nextPoint = new Point((int) (getWidth() - (pointId * pointDist)), (int) translateValue(d.doubleValue()));
						if(pointId != 0) {
							g2d.drawLine(point.x, point.y, nextPoint.x, nextPoint.y);
						}
						pointId++;
						point = nextPoint;
					}
				}
			}
		}

		

		public void mouseDragged(MouseEvent e) {
			if(mClickPosition == null) {
				mClickPosition = e.getPoint();
			}
			else {
				double verticalOffset = mVerticalOffsetRoot - (((mClickPosition.y - e.getY()) / ((double) (getHeight() - 20) / 2.0)) / mScale); 
				setVerticalOffset(verticalOffset);
			}
		}

		public void mouseMoved(MouseEvent e) {
			mClickPosition = null;
		}

		public void mouseWheelMoved(MouseWheelEvent e) {
			
			int count = e.getWheelRotation();
			double scale = mScale;
			if(count < 0) {
				scale *= (count * -1.5);
				scale = Math.min(scale, 10000000.0);
			}
			else {
				if(scale > 0.000002) {
					scale /= (count * 1.5);
				}
			}
			setScale(scale);
		}

		public void mouseClicked(MouseEvent arg0) {
		}

		public void mouseEntered(MouseEvent arg0) {
		}

		public void mouseExited(MouseEvent arg0) {
		}

		public void mousePressed(MouseEvent arg0) {
			mVerticalOffsetRoot = mVerticalOffset;
		}

		public void mouseReleased(MouseEvent arg0) {
			mClickPosition = null;
		}
		
		
		
		
	}



}
