/**
 * 
 * @author Christian Rempis
 * c.rempis@gmx.de
 * 
 */
package de.uos.ikw.octavio.spinalcordmanager.gui.rawcommandpanel;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import de.uos.ikw.octavio.spinalcordmanager.core.SpinalCordManager;
import de.uos.ikw.octavio.spinalcordmanager.gui.GuiElement;
import de.uos.ikw.octavio.spinalcordmanager.gui.util.AutoRepeatIndicator;
import de.uos.ikw.octavio.spinalcordmanager.io.CommandWriter;

public class RawCommandPanel extends GuiElement {

	private JPanel mDisplay;
	private JButton mSendRawCommand;
	private JCheckBox mVerifyCheckBox;
	private JCheckBox mDiscardCheckBox;
	private JTextField mExpirationCheckBox;
	private JButton mEnableAutoRepeatButton;
	private JButton mDisableAutoRepeatButton;
	private JButton mHelpButton;
	private AutoRepeatIndicator mAutoRepeatIndicator;
	private JCheckBox mSendAsDeploymentCommand;
	private JComboBox mCommandField;
	
	public RawCommandPanel() {
		super("RawCommandPanel");
		
		mDisplay = new JPanel(new BorderLayout());
		
		mCommandField = new JComboBox();
		mCommandField.setEditable(true);
		{
			File file = new File("commandHistory.txt");
			if(file.exists()) {
				try {
					BufferedReader reader = new BufferedReader(new FileReader(file));
					String line = reader.readLine();
					while(line != null) {
						mCommandField.addItem(line);
						line = reader.readLine();
					}
				} catch (FileNotFoundException e) {
					e.printStackTrace();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		
		mSendRawCommand = new JButton("Send Raw Command");
		mSendRawCommand.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				JTextField editor = ((JTextField) mCommandField.getEditor().getEditorComponent());
				String line = editor.getText();
				
				boolean ok = sendCommand(line);
				if(ok) {
					editor.setBackground(Color.WHITE);
					
					//add to combo box
					boolean found = false;
					for(int i = 0; i < mCommandField.getItemCount(); ++i) {
						if(mCommandField.getItemAt(i).equals(line)) {
							found = true;
							break;
						}
					}
					if(!found) {
						mCommandField.addItem(line);
						
						//write history to file
						try {
							File file = new File("commandHistory.txt");
							BufferedWriter writer = new BufferedWriter(new FileWriter(file));
							for(int i = 0; i < mCommandField.getItemCount(); ++i) {
								writer.write(mCommandField.getItemAt(i).toString() + "\n");
							}
							writer.close();
						} catch (IOException e) {
							e.printStackTrace();
						}
					}
				}
				else {
					editor.setBackground(Color.RED);
				}
			}
		});
		
		mEnableAutoRepeatButton = new JButton("Auto Repeat ON");
		mEnableAutoRepeatButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				SpinalCordManager.getInstance().getCommandWriter().setAutoRepeat(true);
			}
		});
		
		mDisableAutoRepeatButton = new JButton("Auto Repeat OFF");
		mDisableAutoRepeatButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				SpinalCordManager.getInstance().getCommandWriter().setAutoRepeat(false);
			}
		});
		
		mHelpButton = new JButton("Help");
		mHelpButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				showHelpScreen();
			}
		});
		
		mSendAsDeploymentCommand = new JCheckBox("Deployment Raw");
		
		mAutoRepeatIndicator = new AutoRepeatIndicator();
		mAutoRepeatIndicator.getDisplay().setPreferredSize(new Dimension(20, 20));
		
		mVerifyCheckBox = new JCheckBox("Verify");
		mDiscardCheckBox = new JCheckBox("Discard");
		mExpirationCheckBox = new JTextField("-1");
		
		Box optionBox = Box.createHorizontalBox();
		optionBox.add(mSendAsDeploymentCommand);
		optionBox.add(mVerifyCheckBox);
		optionBox.add(mDiscardCheckBox);
		optionBox.add(mExpirationCheckBox);
		optionBox.add(mEnableAutoRepeatButton);
		optionBox.add(mAutoRepeatIndicator.getDisplay());
		optionBox.add(mDisableAutoRepeatButton);
		optionBox.add(mHelpButton);
	
		JPanel panel = new JPanel(new BorderLayout());
		panel.add(mCommandField, BorderLayout.CENTER);
		panel.add(mSendRawCommand, BorderLayout.EAST);
		mDisplay.add(panel, BorderLayout.NORTH);
		mDisplay.add(optionBox, BorderLayout.SOUTH);
		
	}
	
	
	public JPanel getDisplay() {
		return mDisplay;
	}

	private boolean sendCommand(String command) {
		String[] data = command.split(",");
		Vector<Integer> comData = new Vector<Integer>();
		for(int i = 0; i < data.length; ++i) {
			try {
				if(data[i].trim().startsWith("w")) {
					//interpret as a single word (2 bytes)
					String word = data[i].trim().substring(1);
					int intValue = Integer.parseInt(word);
					if(intValue > 65535) {
						intValue = 65535;
					}
					if(intValue < 0) {
						intValue = 0;
					}
					int firstByte = (0xFF & intValue);
					int secondByte = (0xFF & (intValue >> 8));
					comData.add(firstByte);
					comData.add(secondByte);
				}
				else {
					//interpret as a single byte.
					comData.add(Integer.parseInt(data[i].trim()));
				}
			} catch (NumberFormatException e) {
				System.out.println("Problem: " + data[i] + "  " + command);
				return false;
			}
		}
		int expirationFrame = -1;
		try {
			expirationFrame = Integer.parseInt(mExpirationCheckBox.getText());
		} 
		catch (NumberFormatException e) {
		}
		mExpirationCheckBox.setText("" + expirationFrame);
		
		int[] dataBytes = new int[comData.size()];
		for(int i = 0; i < comData.size(); ++i) {
			dataBytes[i] = comData.elementAt(i);
			//System.out.print("," + dataBytes[i]);
		}
		//System.out.println("");
		
		CommandWriter cw = SpinalCordManager.getInstance().getCommandWriter();
		cw.sendCommand(dataBytes, mVerifyCheckBox.isSelected(), 
						mDiscardCheckBox.isSelected(), expirationFrame,
						mSendAsDeploymentCommand.isSelected()); 
		return true;
	}
	
	private void showHelpScreen() {
		String message = "Please enter the raw command in the following format:\n\n" +
				"<CID>,<targetNo>,<commandNo>,<dataSize>,<data0>,<data1>,...,<dataN>\n\n" +
				"The rest of the command is automatically added to fit the octavio syntax.\n" +
				"You can use 'w' to interpret a number as a word (2 bytes), so" +
				"'w5039' will be parsed to '175,19'. ";
		JOptionPane.showMessageDialog(mDisplay, message);
	}
}



