package de.uos.ikw.octavio.spinalcordmanager.gui.sensormapping;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;
import java.util.logging.Logger;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.ScrollPaneConstants;


import de.uos.ikw.octavio.spinalcordmanager.core.DataListener;
import de.uos.ikw.octavio.spinalcordmanager.core.DataManager;
import de.uos.ikw.octavio.spinalcordmanager.core.SpinalCordManager;
import de.uos.ikw.octavio.spinalcordmanager.formatter.DataFormatter;
import de.uos.ikw.octavio.spinalcordmanager.formatter.NormalizedDoubleFormatter;
import de.uos.ikw.octavio.spinalcordmanager.gui.GuiElement;
import de.uos.ikw.octavio.spinalcordmanager.gui.tableviewer.CellSelectionListener;
import de.uos.ikw.octavio.spinalcordmanager.io.Command;
import de.uos.ikw.octavio.spinalcordmanager.io.CommandWriter;
import de.uos.ikw.octavio.spinalcordmanager.io.ErrorStreamListener;

public class SensorMappingPanel extends GuiElement implements DataListener, 
					ErrorStreamListener, CellSelectionListener 
{

	private JPanel mDisplay;
	private JTextField mRangeMin;
	private JTextField mRangeMax;
	private JButton mSetNormalizedRangeButton;
	private JButton mSetInputRangeButton;
	private JToggleButton mAutoSetRangeButton;
	private JButton mExportAllMappingButton;
	private JButton mWriteMappingToKernelButton;
	private JButton mReadMappingFromKernelButton;
	private JButton mWriteMappingToEEPROMButton;
	private JButton mReadMappingFromEEPROMButton;
	private Vector<Integer> mSelectedIndices = new Vector<Integer>();
	private JTextArea mSelectedIndicesTextField;
	private JButton mInvertInput;
	private JButton mActivateMappingButton;
	private JButton mDeactivateMappingButton;
	
	private Vector<JCheckBox> mCheckBoxes = new Vector<JCheckBox>();
	
	public SensorMappingPanel() {
		super("SensorMappingPanel");
		
		mDisplay = new JPanel(new BorderLayout());
		
		JLabel minLabel = new JLabel("Range Min");
		JLabel maxLabel = new JLabel("Range Max");
		
		mRangeMin = new JTextField(6);
		mRangeMax = new JTextField(6);
		JPanel p = new JPanel();
		p.setPreferredSize(new Dimension(100, 1000));
		
		
		mSetNormalizedRangeButton = new JButton("Set Norm Range");
		mSetNormalizedRangeButton.setMargin(new Insets(1, 1, 1, 1));
		mSetNormalizedRangeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				changeNormalizedRange();
			}
		});
		
		mSetInputRangeButton = new JButton("Set Input Range");
		mSetInputRangeButton.setMargin(new Insets(1, 1, 1, 1));
		mSetInputRangeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				changeInputRange();
			}
		});
		
		
		JPanel rangePanel = new JPanel(new GridLayout(3, 2));
		rangePanel.add(minLabel);
		rangePanel.add(mRangeMin);
		rangePanel.add(maxLabel);
		rangePanel.add(mRangeMax);
		rangePanel.add(mSetNormalizedRangeButton);
		rangePanel.add(mSetInputRangeButton);
		
		Box box = Box.createVerticalBox();
		box.add(rangePanel);
		box.add(p);
		
		
		
		mAutoSetRangeButton = new JToggleButton("Auto Fit Range");
		mAutoSetRangeButton.setMargin(new Insets(1, 1, 1, 1));
		mAutoSetRangeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mAutoSetRangeButton.isSelected()) {
					startAutoFitRange();
				}
				else {
					stopAutoFitRange();
				}
			}
		});
		
		mExportAllMappingButton = new JButton("Export");
		mExportAllMappingButton.setMargin(new Insets(1, 1, 1, 1));
		mExportAllMappingButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				DataFormatter[] formatters = SpinalCordManager.getInstance().getDataManager().getDataFormatters();
				try {
					FileWriter fw = new FileWriter("mapping.txt");
					for(int i = 0; i < formatters.length; ++i) {
						if(formatters[i] instanceof NormalizedDoubleFormatter) {
							NormalizedDoubleFormatter df = (NormalizedDoubleFormatter) formatters[i];
							fw.write("Cell " + i + ": " + df.getMin() + "," + df.getMax() + "\n");
						}
					}
					fw.flush();
					fw.close();
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
		});
		
		mWriteMappingToKernelButton = new JButton("Write to Kernel");
		mWriteMappingToKernelButton.setMargin(new Insets(1, 1, 1, 1));
		mWriteMappingToKernelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				uploadMapping();
			}
		});
		
		mReadMappingFromKernelButton = new JButton("Read from Kernel");
		mReadMappingFromKernelButton.setMargin(new Insets(1, 1, 1, 1));
		mReadMappingFromKernelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				CommandWriter writer = SpinalCordManager.getInstance().getCommandWriter();
				byte msg[] = new byte[3];
				msg[0] = 'm';
				msg[1] = 'r';
				for(int i = 1; i <= 8; ++i) {
					if(mCheckBoxes.elementAt(i - 1).isSelected() || mCheckBoxes.elementAt(8).isSelected()) {
						msg[2] = (byte) i;
						writer.sendMessage(msg);
					}
				}
			}
		});
		
		mWriteMappingToEEPROMButton = new JButton("Write to EEPROM");
		mWriteMappingToEEPROMButton.setMargin(new Insets(1, 1, 1, 1));
		mWriteMappingToEEPROMButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				//TODO
			}
		});
		
		mReadMappingFromEEPROMButton = new JButton("Read from EEPROM");
		mReadMappingFromEEPROMButton.setMargin(new Insets(1, 1, 1, 1));
		mReadMappingFromEEPROMButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				//TODO
			}
		});
		
		mInvertInput = new JButton("Toggle Input Mapping");
		mInvertInput.setMargin(new Insets(1, 1, 1, 1));
		mInvertInput.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				flipInputRange();
			}
		});
		
		mActivateMappingButton = new JButton("EEPROM Mapping ON");
		mActivateMappingButton.setMargin(new Insets(1, 1, 1, 1));
		mActivateMappingButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int[] message = new int[4];
				message[0] = Command.nextCommandId();
				message[1] = 255;
				message[2] = 80; //sensor mapping on
				message[3] = 0;
				for(int i = 0; i < 2; ++i) { //TODO check why command has to be sent twice
					SpinalCordManager.getInstance().getCommandWriter()
						.sendCommand(message, false,	false, -1, false);
				}
			}
		});
		
		
		mDeactivateMappingButton = new JButton("EEPROM Mapping OFF");
		mDeactivateMappingButton.setMargin(new Insets(1, 1, 1, 1));
		mDeactivateMappingButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int[] message = new int[4];
				message[0] = Command.nextCommandId();
				message[1] = 255;
				message[2] = 64; //sensor mapping off
				message[3] = 0;
				for(int i = 0; i < 2; ++i) { //TODO check why command has to be sent twice
					SpinalCordManager.getInstance().getCommandWriter()
					  .sendCommand(message, false,	false, -1, false);
				}
			}
		});
		
		mSelectedIndicesTextField = new JTextArea();
		mSelectedIndicesTextField.setEditable(false);
		JScrollPane scrollPane = new JScrollPane(mSelectedIndicesTextField,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		
		
		
		JPanel box2 = new JPanel(new GridLayout(9, 1));
		box2.add(mInvertInput);
		box2.add(mAutoSetRangeButton);
		box2.add(mExportAllMappingButton);
		box2.add(mReadMappingFromKernelButton);
		box2.add(mWriteMappingToKernelButton);
		box2.add(mReadMappingFromEEPROMButton);
		box2.add(mWriteMappingToEEPROMButton);
		box2.add(mActivateMappingButton);
		box2.add(mDeactivateMappingButton);
		
		JPanel legSelectionPanel = new JPanel(new GridLayout(3, 3));
		legSelectionPanel.setBorder(BorderFactory.createTitledBorder("Apply for selected legs."));
		for(int i = 1; i <= 8; ++i) {
			JCheckBox checkBox = new JCheckBox("  " + i);
			mCheckBoxes.add(checkBox);
			legSelectionPanel.add(checkBox);
		}
		JCheckBox allCheckBox = new JCheckBox("all ");
		allCheckBox.setSelected(true);
		mCheckBoxes.add(allCheckBox);
		legSelectionPanel.add(allCheckBox);
		
		box.add(legSelectionPanel);
		
		mDisplay.add(box, BorderLayout.WEST);
		mDisplay.add(box2, BorderLayout.EAST);
		mDisplay.add(scrollPane, BorderLayout.CENTER);
		
		SpinalCordManager.getInstance().getErrorReader().addErrorStreamListener(this);
		
	}

	
	public JPanel getDisplay() {
		return mDisplay;
	}
	
	
	
	public void dataContentChanged() {
	
		byte[] data = SpinalCordManager.getInstance().getDataManager().getLastInterpretedDataFrame();
		int index = 0;
		
		DataFormatter[] formatters = SpinalCordManager.getInstance().getDataManager().getDataFormatters();
		for(int i = 0; i < formatters.length; ++i) {
			if(formatters[i] instanceof NormalizedDoubleFormatter) {
				NormalizedDoubleFormatter cell = ((NormalizedDoubleFormatter) formatters[i]);
				double min = cell.getMin();
				double max = cell.getMax();
				double value = cell.getUnnormalizedDouble(data, index);
				
				if(value < min) {
					cell.setRange(value, cell.getMax());
				}
				if(value > max) {
					cell.setRange(cell.getMin(), value);
				}
			}
			index += formatters[i].getNumberOfBytesPerData();
		}
	}


	public void dataHistoryChanged() {
	}


	public void dataStructureChanged() {
	}
	
	
	


	public void receivedDataMessage(String messageHeader, byte[] data) {
		
		//handle only data messages of type [B][Mx]
		if(messageHeader == null || !messageHeader.startsWith("[M") || messageHeader.length() != 4) {
			//TODO remove
			System.out.println("Could not handle messageHeader " + messageHeader + " " + messageHeader.length());
			return;
		}
		
		if(data == null || data.length < 4 * 11) {
			Logger.getLogger("").warning("Could not interpret mapping data. Too few bytes read.");
			return;
		}
		
		int legNumber = 0;
		try {
			legNumber = Integer.parseInt(messageHeader.substring(2, 3));
		}
		catch(NumberFormatException e) {
			Logger.getLogger("").warning("Could not determine target leg for mapping data. " + messageHeader);
			return;
		}
		if(legNumber < 1 | legNumber > 8) {
			Logger.getLogger("").warning("Leg number out of range while receiving mapping data. " + messageHeader + " " + legNumber);
			return;
		}
		
		DataManager dm = SpinalCordManager.getInstance().getDataManager();
		DataFormatter[] formatters = dm.getDataFormatters();
		
		int index = 0;
		int bytes = 0;
		
		//find starting position for leg0
		int startingByte = dm.getMasterSectionSize() + dm.getSlot0SectionSize();
		for(; bytes < startingByte;) {
			if(index >= formatters.length) {
				Logger.getLogger("").warning("Not enough formatter objects found!");
				return;
			}
			bytes += formatters[index].getNumberOfBytesPerData();
			index++;
		}
		
		for(int i = 1; i <= 8; ++i) {

			for(int j = 0; j < 6; ++j) {
				if(index >= formatters.length) {
					Logger.getLogger("").warning("Not enough formatter objects found!");
					return;
				}
				bytes += formatters[index].getNumberOfBytesPerData();
				index++;
			}
			//sensor data
			for(int j = 0; j < 11; ++j) {
				if(index >= formatters.length) {
					Logger.getLogger("").warning("Not enough formatter objects found!");
					return;
				}
				if(i == legNumber) {
					DataFormatter df = formatters[index];
					if(df instanceof NormalizedDoubleFormatter && data.length >= (j * 4) + 4) {
						NormalizedDoubleFormatter ndf = (NormalizedDoubleFormatter) df;
						double min = (double) ((int) ((0xFF & (int) data[j * 4 + 1]) << 8 | (0xFF & (int) data[j * 4])));
						double max = (double) ((int) ((0xFF & (int) data[j * 4 + 3]) << 8 | (0xFF & (int) data[j * 4 + 2])));
						ndf.setRange(min, max);
					}
					else {
						System.out.println("DataFormatter was not a NormalizedDoubleFormatter");
					}
				}
				bytes += formatters[index].getNumberOfBytesPerData();
				index++;
			}
			if(i == legNumber) {
				dm.setRawData(dm.getLastInterpretedDataFrame());			
				return;
			}
			
			for(int j = 0; j < 10; ++j) {
				if(index >= formatters.length) {
					Logger.getLogger("").warning("Not enough formatter objects found!");
					return;
				}
				bytes += formatters[index].getNumberOfBytesPerData();
				index++;
			}
		}
	}


	public void receivedErrorMessage(String errorMessage) {
	}
	
	


	public void selectionChanged(int[] selectedIndices) {
		StringBuffer text = new StringBuffer();
		mSelectedIndices.clear();
		for(int i = 0; i < selectedIndices.length; ++i) {
			text.append(selectedIndices[i] + "\n");
			mSelectedIndices.add(selectedIndices[i]);
			DataFormatter[] formatters = SpinalCordManager.getInstance().getDataManager().getDataFormatters();
			if(selectedIndices[i] >= formatters.length) {
				Logger.getLogger("").warning("Index too high! " + selectedIndices[i] + " max: " + (formatters.length - 1));
				continue;
			}
			DataFormatter df = formatters[selectedIndices[i]];
			if(df instanceof NormalizedDoubleFormatter) {
				NormalizedDoubleFormatter ndf = (NormalizedDoubleFormatter) df;
				mRangeMin.setText("" + ndf.getMin());
				mRangeMax.setText("" + ndf.getMax());
			}
		}
		mSelectedIndicesTextField.setText(text.toString());
	}


	private void changeInputRange() {
		try {
			double min = Double.parseDouble(mRangeMin.getText());
			double max = Double.parseDouble(mRangeMax.getText());
			
			//if(min < max) { //TODO check if this should be allowed
			if(true) {
			
				mRangeMin.setBackground(Color.white);
				mRangeMax.setBackground(Color.white);
				
				DataFormatter[] formatters = SpinalCordManager.getInstance().getDataManager().getDataFormatters();
				for(int i = 0; i < formatters.length; ++i) {
					boolean isSelected = false;
					for(int j = 0; j < mSelectedIndices.size(); ++j) {
						if(i == mSelectedIndices.elementAt(j)) {
							isSelected = true;
							break;
						}
					}
					if(isSelected) {
						if(formatters[i] instanceof NormalizedDoubleFormatter) {
							((NormalizedDoubleFormatter) formatters[i]).setRange(min, max);
						}
					}
				}
				DataManager dm = SpinalCordManager.getInstance().getDataManager();
				dm.setRawData(dm.getLastInterpretedDataFrame());
				return;
			}
		} catch (NumberFormatException e) {
		}
		mRangeMin.setBackground(Color.red);
		mRangeMax.setBackground(Color.red);
	}
	
	private void changeNormalizedRange() {
		try {
			double min = Double.parseDouble(mRangeMin.getText());
			double max = Double.parseDouble(mRangeMax.getText());
			
			//if(min < max) { //TODO should be allowed?
			if(true) {
			
				mRangeMin.setBackground(Color.white);
				mRangeMax.setBackground(Color.white);
				
				DataFormatter[] formatters = SpinalCordManager.getInstance().getDataManager().getDataFormatters();
				
				
				for(int i = 0; i < formatters.length; ++i) {
					boolean isSelected = false;
					for(int j = 0; j < mSelectedIndices.size(); ++j) {
						if(i == mSelectedIndices.elementAt(j)) {
							isSelected = true;
							break;
						}
					}
					if(isSelected) {
						if(formatters[i] instanceof NormalizedDoubleFormatter) {
							((NormalizedDoubleFormatter) formatters[i]).setNormalizedRange(min, max);
						}
					}
				}
				DataManager dm = SpinalCordManager.getInstance().getDataManager();
				dm.setRawData(dm.getLastInterpretedDataFrame());
				return;
			}
		} catch (NumberFormatException e) {
		}
		mRangeMin.setBackground(Color.red);
		mRangeMax.setBackground(Color.red);
	}
	
	private void flipInputRange() {
			
		DataFormatter[] formatters = SpinalCordManager.getInstance().getDataManager().getDataFormatters();
		for(int i = 0; i < formatters.length; ++i) {
			boolean isSelected = false;
			int selectedIndex = 0;
			for(int j = 0; j < mSelectedIndices.size(); ++j) {
				if(i == mSelectedIndices.elementAt(j)) {
					isSelected = true;
					selectedIndex = j;
					break;
				}
			}
			if(isSelected) {
				if(formatters[i] instanceof NormalizedDoubleFormatter) {
					NormalizedDoubleFormatter ndf = (NormalizedDoubleFormatter) formatters[i];
					//flip input min and max
					ndf.setRange(ndf.getMax(), ndf.getMin());
					
					if(selectedIndex == mSelectedIndices.size() - 1) {
						//update the range text field with the values of the last selected formatter.
						mRangeMin.setText("" + ndf.getMin());
						mRangeMax.setText("" + ndf.getMax());
					}
				}
			}
		}
		
		
		
		DataManager dm = SpinalCordManager.getInstance().getDataManager();
		dm.setRawData(dm.getLastInterpretedDataFrame());
		return;

	}
	
	private void startAutoFitRange() {
		mInvertInput.setEnabled(false);
		
		DataManager dm = SpinalCordManager.getInstance().getDataManager();
		DataFormatter[] formatters = dm.getDataFormatters();
		
		for(int i = 0; i < formatters.length; ++i) {
			
			int column = i / dm.getNumberOfRows();
			int row = i % dm.getNumberOfRows();
			
			//make sure that we start auto mapping only for the sensors of selected legs.
			if(column > 0 && column <= 8 && row >= 6 && row <= 16) {
				if(mCheckBoxes.elementAt(column - 1).isSelected() || mCheckBoxes.elementAt(8).isSelected()) {
					if(formatters[i] instanceof NormalizedDoubleFormatter) {
						((NormalizedDoubleFormatter) formatters[i]).setRange(Double.MAX_VALUE, Double.MIN_VALUE);
					}
				}
			}
		}
		SpinalCordManager.getInstance().getDataManager().addDataListener(this);
	}
	
	private void stopAutoFitRange() {
		SpinalCordManager.getInstance().getDataManager().removeDataListener(this);	
		mInvertInput.setEnabled(true);
	}
	
	
	private void uploadMapping() {
		DataManager dm = SpinalCordManager.getInstance().getDataManager();
		DataFormatter[] formatters = dm.getDataFormatters();
		
		int size = (11 * 2 * 2) + 3;
		byte[] message = new byte[size];
		message[0] = 'm';
		message[1] = 'u';
		
		
		int index = 0;
		int bytes = 0;
		int messageIndex = 3;
		
		//find starting position for leg0
		int startingByte = dm.getMasterSectionSize() + dm.getSlot0SectionSize();
		for(; bytes < startingByte;) {
			if(index >= formatters.length) {
				Logger.getLogger("").warning("Not enough formatter objects found!");
				return;
			}
			bytes += formatters[index].getNumberOfBytesPerData();
			index++;
		}
		
		for(int i = 0; i < 8; ++i) {
			message[2] = (byte) (i + 1);
			messageIndex = 3;
			
			for(int j = 0; j < 6; ++j) {
				if(index >= formatters.length) {
					Logger.getLogger("").warning("Not enough formatter objects found!");
					return;
				}
				bytes += formatters[index].getNumberOfBytesPerData();
				index++;
			}
			for(int j = 0; j < 11; ++j) {
				if(index >= formatters.length) {
					Logger.getLogger("").warning("Not enough formatter objects found!");
					return;
				}
				DataFormatter df = formatters[index];
				double min = 0.0;
				double max = 0.0;
				if(df instanceof NormalizedDoubleFormatter) {
					NormalizedDoubleFormatter ndf = (NormalizedDoubleFormatter) df;
					min = ndf.getMin();
					max = ndf.getMax();
				}
				else {
					System.out.println("DataFormatter was not a NormalizedDoubleFormatter");
				}
				if(messageIndex >= message.length) {
					Logger.getLogger("").warning("Message size was not large enough.");
					return;
				}
				byte hi = (byte) (((int) min) >> 8);
				byte low = (byte) ((int) min);
				message[messageIndex] = low;
				message[messageIndex + 1] = hi;
				
				hi = (byte) (((int) max) >> 8);
				low = (byte) ((int) max);
				message[messageIndex + 2] = low;
				message[messageIndex + 3] = hi;
				
				messageIndex += 4;
				
				bytes += formatters[index].getNumberOfBytesPerData();
				index++;
			}
			
			//send mapping data of this leg.
			if(mCheckBoxes.elementAt(i).isSelected() || mCheckBoxes.elementAt(8).isSelected()) {
				SpinalCordManager.getInstance().getCommandWriter().sendMessage(message);
			}
			/*
			for(int j = 0; j < message.length; ++j) {
				System.out.print("," + (int) (0xff & (int) message[j]));
			}
			System.out.println(".");
			*/
			
			for(int j = 0; j < 10; ++j) {
				if(index >= formatters.length) {
					Logger.getLogger("").warning("Not enough formatter objects found!");
					return;
				}
				bytes += formatters[index].getNumberOfBytesPerData();
				index++;
			}
		}
		
		
	}

}
