/**
 * 
 * @author Christian Rempis
 * c.rempis@gmx.de
 * 
 */
package de.uos.ikw.octavio.spinalcordmanager.gui.tableviewer;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;

import de.uos.ikw.octavio.spinalcordmanager.core.DataListener;
import de.uos.ikw.octavio.spinalcordmanager.core.DataManager;
import de.uos.ikw.octavio.spinalcordmanager.core.SpinalCordManager;
import de.uos.ikw.octavio.spinalcordmanager.formatter.DataFormatter;
import de.uos.ikw.octavio.spinalcordmanager.formatter.NormalizedDoubleFormatter;
import de.uos.ikw.octavio.spinalcordmanager.gui.GuiElement;

public class TableViewer extends GuiElement implements DataListener, ListSelectionListener {

	private JPanel mDisplay;
	private JScrollPane mScrollPane;
	private JTable mTable;
	private AbstractTableModel mTableModel;
	private boolean mRepaint;
	private Timer mRepaintTimer;
	
	private static final String mModeValue = "Value";
	private static final String mModeBytes = "Bytes";
	private static final String mModeHex = "Hex";
	//private static final String mModeMax = "Max Mapping";
	//private static final String mModeMin = "Min Mapping";
	private static final String mModeMinMax = "Range Input Mapping";
	private static final String mModeNormalizedMinMax = "Range Normalized Mapping";
	private static final String mNodeIgnoreNormalization = "Raw Integer";
	
	private JComboBox mModeChooser;
	private Vector<CellSelectionListener> mCellSelectionListeners = new Vector<CellSelectionListener>();
	
	public TableViewer(String name) {
		super(name);
		
		SpinalCordManager.getInstance().getDataManager().addDataListener(this);
		
		mDisplay = new JPanel(new BorderLayout());
		
		mTableModel = new SpinalCordDataModel();
		
		mTable = new JTable(mTableModel);
		mTable.setCellSelectionEnabled(true);
		mTable.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
		mTable.setDragEnabled(false);
		mScrollPane = new JScrollPane(mTable);
		mTable.getSelectionModel().addListSelectionListener(this);
		mTable.getColumnModel().getColumn(0).setMaxWidth(40);
		
		
		String[] modes = {mModeValue, mModeBytes, mModeHex, mNodeIgnoreNormalization,
							mModeMinMax, mModeNormalizedMinMax};
		mModeChooser = new JComboBox(modes);
		mModeChooser.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				int mode = 0;
				if(mModeChooser.getSelectedItem().toString().equals(mModeValue)) {
					mode = DataFormatter.MODE_VALUE;
				}
				else if(mModeChooser.getSelectedItem().toString().equals(mModeBytes)) {
					mode =DataFormatter.MODE_BYTE;
				}
				else if(mModeChooser.getSelectedItem().toString().equals(mModeHex)) {
					mode = DataFormatter.MODE_HEX;
				}
				else if(mModeChooser.getSelectedItem().toString().equals(mModeMinMax)) {
					mode = NormalizedDoubleFormatter.MODE_MINMAX;
				}
				else if(mModeChooser.getSelectedItem().toString().equals(mModeNormalizedMinMax)) {
					mode = NormalizedDoubleFormatter.MODE_NORMALIZED_MINMAX;
				}
				else if(mModeChooser.getSelectedItem().toString().equals(mNodeIgnoreNormalization)) {
					mode = NormalizedDoubleFormatter.MODE_IGNORE_NORMALIZATION;
				}
				DataFormatter[] formatter = SpinalCordManager.getInstance().getDataManager().getDataFormatters();
				for(int i = 0; i < formatter.length; ++i) {
					formatter[i].setFormatMode(mode);
				}
				DataManager dm = SpinalCordManager.getInstance().getDataManager();
				dm.setRawData(dm.getLastInterpretedDataFrame());
				mTable.repaint();
			}
			
		});
		
		mDisplay.add(mScrollPane, BorderLayout.CENTER);
		mDisplay.add(mModeChooser, BorderLayout.SOUTH);
		
		mTable.setDefaultRenderer(Object.class, new DefaultTableCellRenderer() {
			
			public static final long serialVersionUID = 284L;
			
			public Component getTableCellRendererComponent(JTable table, Object value, 
							boolean isSelected, boolean hasFocus, int row, int column) 
			{
				if(column == 0) {
					setBackground(Color.lightGray);
				}
				else {
					DataManager dm = SpinalCordManager.getInstance().getDataManager();
					Color c = dm.getDataColors()[(column - 1) * dm.getNumberOfRows() + row];
					if(c != null) {
						setBackground(c);
					}
					else {
						setBackground(Color.WHITE);
					}
				}
				return super.getTableCellRendererComponent(table, value, isSelected, 
															hasFocus, row, column);
			}
		});
		
		mRepaintTimer = new Timer();
		mRepaintTimer.schedule(new RepaintTimerTask(), 500, 500);
	}
	
	public JPanel getDisplay() {
		return mDisplay;
	}
	
	public void valueChanged(ListSelectionEvent e) {
		if(e.getValueIsAdjusting()) {
			return;
		}
		Vector<Integer> selectedIndices = new Vector<Integer>();
		for(int column = 1; column < mTable.getColumnCount(); ++column) {
			for(int row = 0; row < mTable.getRowCount(); ++row) {
				if(mTable.isCellSelected(row, column)) {
					selectedIndices.add((column - 1) * mTable.getRowCount() + row);
				}
			}
		}
		int indices[] = new int[selectedIndices.size()];
		for(int i = 0; i < selectedIndices.size(); ++i) {
			indices[i] = selectedIndices.elementAt(i);
		}
		notifyCellSelectionListeners(indices);
		
		/*
		int colIndex = mTable.getSelectedColumn();
		int rowIndex = mTable.getSelectedRow();
		
		if(colIndex > 0 && rowIndex >= 0) {
			int[] indices = new int[1];
			indices[0] = (colIndex - 1) * mTable.getRowCount() + rowIndex;
			notifyCellSelectionListeners(indices);
		}
		*/
		
	}
	
	public JTable getTable() {
		return mTable;
	}

	public void dataContentChanged() {
		mRepaint = true;
//		DataManager dm = SpinalCordManager.getInstance().getDataManager();
//		System.out.println("Frames: Rec: " + dm.getLastReceivedFrameNumber() + " Int: " + dm.getLastInterpretedFrameNumber());
	}

	public void dataStructureChanged() {
		mRepaint = true;
	}
	
	public void dataHistoryChanged() {
	}
	
	public boolean addCellSelectionListener(CellSelectionListener listener) {
		if(listener == null || mCellSelectionListeners.contains(listener)) {
			return false;
		}
		mCellSelectionListeners.add(listener);
		return true;
	}
	
	public boolean removeCellSelectionListener(CellSelectionListener listener) {
		if(listener == null || !mCellSelectionListeners.contains(listener)) {
			return false;
		}
		mCellSelectionListeners.add(listener);
		return true;
	}
	
	public void notifyCellSelectionListeners(int[] selectedIndices) {
		for(Iterator<CellSelectionListener> i = mCellSelectionListeners.iterator(); i.hasNext();) {
			i.next().selectionChanged(selectedIndices);
		}
	}
	
	public Vector<CellSelectionListener> getCellSelectionListeners() {
		return new Vector<CellSelectionListener>(mCellSelectionListeners);
	}
	
	public class RepaintTimerTask extends TimerTask {

		public void run() {
			if(mRepaint) {
				mRepaint = false;
				mTable.repaint();
			}
		}
		
	}
	
	
}
