/**
 * 
 * @author Christian Rempis
 * c.rempis@gmx.de
 * 
 */
package de.uos.ikw.octavio.spinalcordmanager.io;

public class Command {
	
	private static int mCommandIds;
	
	private int[] mData;
	private boolean mVerifySending;
	private boolean mDiscardIfNotSendImmediately;
	private int mExpirationFrameNumber;
	private boolean mDeploymentMode;
	
	public Command(int[] data, boolean verifySending, boolean discardIfNotSendImmediately) 
	{
		this(data, verifySending, discardIfNotSendImmediately, -1, false);
	}
		
	public Command(int[] data, boolean verifySending, boolean discardIfNotSendImmediately,
						boolean deploymentMode) 
	{
		this(data, verifySending, discardIfNotSendImmediately, -1, deploymentMode);
	}
	
	public Command(int[] data, boolean verifySending, boolean discardIfNotSendImmediately,
			int expirationFrameNumber) 
	{
		this(data, verifySending, discardIfNotSendImmediately, expirationFrameNumber, false);
	}
	
	public Command(int[] data, boolean verifySending, boolean discardIfNotSendImmediately,
			int expirationFrameNumber, boolean deploymentMode) 
	{
		mData = data.clone();
		mVerifySending = verifySending;
		mDiscardIfNotSendImmediately = discardIfNotSendImmediately;
		mExpirationFrameNumber = expirationFrameNumber;
		mDeploymentMode = deploymentMode;
	}
	
	/**
	 * Returns the internal data array.
	 * @return the internal data array.
	 */
	public int[] getData() {
		return mData;
	}
	
	/**
	 * Returns true, if the message is sent until its id was acknowledged 
	 * by the SpinalCord.
	 * 
	 * @return
	 */
	public boolean requiresVerifiedSending() {
		return mVerifySending;
	}
	
	/**
	 * Returns true, if this message is send in discard mode. In this mode 
	 * messages are discarded, if they could not be sent in the next possible 
	 * frame. If not in this mode, messages are kept until it could be sent 
	 * in the correct time slot to the spinal cord. This does not guarantee 
	 * that the message is also received by the spinal cord! For this used 
	 * the message verification.
	 * 
	 * @return
	 */
	public boolean isDiscardedIfNotSendImmediately() {
		return mDiscardIfNotSendImmediately;
	}
	
	public int getExpirationFrameNumber() {
		return mExpirationFrameNumber;
	}
	
	public boolean isSendInDeploymentMode() {
		return mDeploymentMode;
	}
	
	public static int nextCommandId() {
		mCommandIds++;
		if(mCommandIds > 255) {
			mCommandIds = 0;
		}
		return mCommandIds;
	}
}
