/**
 * 
 * @author Christian Rempis
 * c.rempis@gmx.de
 * 
 */
package de.uos.ikw.octavio.spinalcordmanager.test;

import de.uos.ikw.octavio.spinalcordmanager.formatter.DataFormatter;
import de.uos.ikw.octavio.spinalcordmanager.formatter.EmptyDataFormatter;
import de.uos.ikw.octavio.spinalcordmanager.formatter.Float8_8DataFormatter;
import de.uos.ikw.octavio.spinalcordmanager.formatter.Int16DataFormatter;
import de.uos.ikw.octavio.spinalcordmanager.formatter.Int8DataFormatter;
import de.uos.ikw.octavio.spinalcordmanager.formatter.SyncDataFormatter;
import junit.framework.TestCase;

public class TestDataFormatters extends TestCase {

	protected void setUp() throws Exception {
	}

	protected void tearDown() throws Exception {
	}

	public void testDataFormatter() {
		DataFormatter f = new DataFormatter("Name", 3, "Desc");
		
		assertEquals(f.getName(), "Name");
		assertEquals(f.getFormatMode(), DataFormatter.MODE_VALUE);
		assertEquals(f.getDescription(), "Desc");
		assertEquals(f.getNumberOfBytesPerData(), 3);
		
		byte[] data = {0x10, 0x55, 0x00};
		
		assertNull(f.formatAsBytes(data, 1));
		assertNull(f.formatAsBytes(data, 100));
		assertNull(f.formatAsBytes(data, -10));
		
		assertNull(f.formatAsHex(data, 1));
		assertNull(f.formatAsHex(data, 100));
		assertNull(f.formatAsHex(data, -10));
		
		assertNotNull(f.formatAsBytes(data, 0));
		assertTrue(f.formatAsBytes(data, 0) instanceof String);
		assertEquals(f.formatAsBytes(data, 0), "16  85  0  ");
		assertNotNull(f.formatAsHex(data, 0));
		assertTrue(f.formatAsHex(data, 0) instanceof String);
		assertEquals(f.formatAsHex(data, 0), "10  55  0  ");
		
		
		assertNull(f.getFormattedData(data, 0));
		f.setFormatMode(DataFormatter.MODE_BYTE);
		assertEquals(f.getFormatMode(), DataFormatter.MODE_BYTE);
		assertEquals(f.getFormattedData(data, 0), "16  85  0  ");
		f.setFormatMode(DataFormatter.MODE_HEX);
		assertEquals(f.getFormatMode(), DataFormatter.MODE_HEX);
		assertEquals(f.getFormattedData(data, 0), "10  55  0  ");
		
	}
	
	public void testSyncDataFormatter() {
		SyncDataFormatter f = new SyncDataFormatter();
		
		assertEquals(f.getName(), "Sync");
		assertEquals(f.getFormatMode(), DataFormatter.MODE_VALUE);
		assertFalse(f.getDescription().equals(""));
		assertEquals(f.getNumberOfBytesPerData(), 1);
		
		byte[] data = {0x10, 0x55, 0x00};
		assertNull(f.formatData(data, 0));
		assertNotNull(f.formatData(data, 1));
		assertTrue(f.formatData(data, 1) instanceof String);
		assertEquals(f.formatData(data, 1), "SYNC");
		assertNotNull(f.formatAsBytes(data, 0));
		assertEquals(f.formatAsBytes(data, 0), "16  ");
		assertNotNull(f.formatAsBytes(data, 1));
		assertEquals(f.formatAsBytes(data, 1), "85  ");
		assertNotNull(f.formatAsHex(data, 0));
		assertEquals(f.formatAsHex(data, 0), "10  ");
		assertNotNull(f.formatAsHex(data, 1));
		assertEquals(f.formatAsHex(data, 1), "55  ");
		
		assertNull(f.formatData(data, 100));
		assertNull(f.formatData(data, -10));
		
	}
	
	public void testInt8DataFormatter() {
		
		Int8DataFormatter f = new Int8DataFormatter();
		
		assertEquals(f.getName(), "Int8");
		assertEquals(f.getFormatMode(), DataFormatter.MODE_VALUE);
		assertFalse(f.getDescription().equals(""));
		assertEquals(f.getNumberOfBytesPerData(), 1);
		
		byte[] data = {0x10, 0x55, 0x00};
		assertNotNull(f.formatData(data, 0));
		assertNotNull(f.formatData(data, 1));
		assertTrue(f.formatData(data, 0) instanceof Integer);
		assertEquals(((Integer) f.formatData(data, 0)).intValue(), 16);
		assertEquals(((Integer) f.formatData(data, 1)).intValue(), 85);
		assertEquals(((Integer) f.formatData(data, 2)).intValue(), 0);
		
		assertNull(f.formatData(data, 100));
		assertNull(f.formatData(data, -10));
	}
	
	public void testInt16DataFormatter() {
		
		Int16DataFormatter f = new Int16DataFormatter();
		
		assertEquals(f.getName(), "Int16");
		assertEquals(f.getFormatMode(), DataFormatter.MODE_VALUE);
		assertFalse(f.getDescription().equals(""));
		assertEquals(f.getNumberOfBytesPerData(), 2);
		
		byte[] data = {0x55, 0x00, 0x10, 0x7a};
		
		assertNotNull(f.formatData(data, 0));
		assertNotNull(f.formatData(data, 1));
		assertNotNull(f.formatData(data, 2));
		assertNull(f.formatData(data, 3));
		assertTrue(f.formatData(data, 0) instanceof Integer);
		assertEquals(((Integer) f.formatData(data, 0)).intValue(), 85);
		assertEquals(((Integer) f.formatData(data, 1)).intValue(), 4096);
		assertEquals(((Integer) f.formatData(data, 2)).intValue(), 31248);
		
		assertNull(f.formatData(data, 100));
		assertNull(f.formatData(data, -10));
	}
	
	public void testEmptyDataFormatter() {
		
		EmptyDataFormatter f0 = new EmptyDataFormatter("Empty0", 0);
		
		assertEquals(f0.getName(), "Empty0");
		assertEquals(f0.getFormatMode(), DataFormatter.MODE_VALUE);
		assertFalse(f0.getDescription().equals(""));
		assertEquals(f0.getNumberOfBytesPerData(), 0);
		
		byte[] data = {0x55, 0x00, 0x10};
		
		assertNull(f0.formatData(data, 0));
		assertNull(f0.formatData(data, 1));
		assertNull(f0.formatData(data, 2));
		assertNull(f0.formatData(data, 3));
		
		assertNull(f0.formatData(data, 100));
		assertNull(f0.formatData(data, -10));
		
		
		EmptyDataFormatter f2 = new EmptyDataFormatter("Empty2", 2);
		
		assertEquals(f2.getName(), "Empty2");
		assertEquals(f2.getFormatMode(), DataFormatter.MODE_VALUE);
		assertFalse(f2.getDescription().equals(""));
		assertEquals(f2.getNumberOfBytesPerData(), 2);
		
	}
	
	
	public void testFloat8_8DataFormatter() {
		
		Float8_8DataFormatter f0 = new Float8_8DataFormatter();
		
		assertEquals(f0.getName(), "Float8/8");
		assertEquals(f0.getFormatMode(), DataFormatter.MODE_VALUE);
		assertFalse(f0.getDescription().equals(""));
		assertEquals(f0.getNumberOfBytesPerData(), 2);
		
		//byte[] data = {0x55, 0x00, 0x10};
		
		//TODO
		
	}
	
}
